#include <config.h>
#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20101229

#define YYEMPTY        (-1)
#define yyclearin      (yychar = YYEMPTY)
#define yyerrok        (yyerrflag = 0)
#define YYRECOVERING() (yyerrflag != 0)

#define YYPREFIX "yy"

#define YYPURE 0

#line 32 "p.y"
  
/*
 * DESCRIPTION
 *   Simple context-free grammar for parsing the control file. 
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Olivier Beyssac, <ob@r14.freenix.org>
 *  @author Kianusch Sayah Karadji <kianusch.sayah.karadji@sk-tech.net>
 *  @author Martin Pala <martinp@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *  @author Rory Toma <rory@digeo.com>
 */

#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif 

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif 

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif 

#ifdef HAVE_GRP_H
#include <grp.h>
#endif 
  
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
  
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
  
#ifdef HAVE_TIME_H
#include <time.h>
#endif
  
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_ASM_PARAM_H
#include <asm/param.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#ifndef HAVE_SOL_IP
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#endif

#ifdef HAVE_NETINET_IP_ICMP_H
#include <netinet/ip_icmp.h>
#endif

#ifdef HAVE_REGEX_H
#include <regex.h>
#endif
  
#include "net.h"
#include "monitor.h"
#include "protocol.h"
#include "engine.h"
#include "alert.h"
#include "process.h"
#include "ssl.h"
#include "device.h"


/* ------------------------------------------------------------- Definitions */

  struct IHavePrecedence {
    int daemon;
    int logfile;
    int pidfile;
  }; 

  struct myrate {
    unsigned count;
    unsigned cycles;
  }; 

  /* yacc interface */
  void  yyerror(const char *,...);
  void  yyerror2(const char *,...);
  void  yywarning(const char *,...);
  void  yywarning2(const char *,...);

  /* lexer interface */
  int yylex(void);
  extern FILE *yyin;
  extern int lineno;
  extern int arglineno;
  extern char *yytext;
  extern char *argyytext;
  extern char *currentfile;
  extern char *argcurrentfile;
  extern int buffer_stack_ptr;

  /* Local variables */
  static int cfg_errflag = FALSE;
  static Service_T tail = NULL;
  static Service_T current = NULL;
  static unsigned int eventset;
  static Request_T urlrequest = NULL;
  static Command_T command = NULL;
  static Command_T command1 = NULL;
  static Command_T command2 = NULL;
  static Service_T depend_list = NULL;
  static struct mygid gidset;
  static struct myuid uidset;
  static struct myperm permset;
  static struct mysize sizeset;
  static struct mymatch matchset;
  static struct myicmp icmpset;
  static struct mymail mailset;
  static struct myport portset;
  static struct mymailserver mailserverset;
  static struct myfilesystem filesystemset;
  static struct myresource resourceset;
  static struct mychecksum checksumset;
  static struct mytimestamp timestampset;
  static struct myactionrate actionrateset;
  static struct IHavePrecedence ihp = {FALSE, FALSE, FALSE};
  static struct myrate rate1 = {1, 1};
  static struct myrate rate2 = {1, 1};
  static char * htpasswd_file = NULL;
  static int    digesttype = DIGEST_CLEARTEXT;
  static int    hassystem = FALSE;

#define BITMAP_MAX (sizeof(long long) * 8)

  
/* -------------------------------------------------------------- Prototypes */

  static void  preparse();
  static void  postparse();
  static void  addservice(Service_T);
  static void  addmail(char *, Mail_T, Mail_T *, unsigned int, unsigned int);
  static void  createservice(int, char *, char *, int (*)(Service_T));
  static void  adddependant(char *);
  static void  addservicegroup(char *);
  static void  addport(Port_T);
  static void  addresource(Resource_T);
  static void  addtimestamp(Timestamp_T, int);
  static void  addactionrate(ActionRate_T);
  static void  addsize(Size_T);
  static void  addfilesystem(Filesystem_T);
  static void  addicmp(Icmp_T);
  static void *addprotocol(int);
  static void  addgeneric(Port_T, char*, char*);
  static void  addcommand(int, unsigned);
  static void  addargument(char *);
  static void  addmmonit(URL_T, int, int, char *);
  static void  addmailserver(MailServer_T);
  static int   addcredentials(char *, char *, int, int);
#ifdef HAVE_LIBPAM
  static void  addpamauth(char *, int);
#endif
  static void  addhtpasswdentry(char *, char *, int);
  static uid_t get_uid(char *, uid_t);
  static gid_t get_gid(char *, gid_t);
  static void  addchecksum(Checksum_T);
  static void  addperm(Perm_T);
  static void  addmatch(Match_T, int, int);
  static void  addmatchpath(Match_T, int);
  static void  adduid(Uid_T);
  static void  addgid(Gid_T);
  static void  addeuid(uid_t);
  static void  addegid(gid_t);
  static void  addeventaction(EventAction_T *, int, int);
  static void  seteventaction(EventAction_T *, int, int);
  static void  prepare_urlrequest(URL_T U);
  static void  seturlrequest(int, char *);
  static void  setlogfile(char *);
  static void  setpidfile(char *);
  static void  reset_mailset();
  static void  reset_mailserverset();
  static void  reset_portset();
  static void  reset_resourceset();
  static void  reset_timestampset();
  static void  reset_actionrateset();
  static void  reset_sizeset();
  static void  reset_checksumset();
  static void  reset_permset();
  static void  reset_uidset();
  static void  reset_gidset();
  static void  reset_filesystemset();
  static void  reset_icmpset();
  static void  reset_rateset();
  static void  check_name(char *);
  static void  check_every(int);
  static int   check_perm(int);
  static void  check_hostname (char *);
  static void  check_exec(char *);
  static int   cleanup_hash_string(char *);
  static void  check_depend();
  static void  setsyslog(char *);
  static Command_T copycommand(Command_T);
  static int verifyMaxForward(int);  

#line 265 "p.y"
#ifdef YYSTYPE
#undef  YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
#endif
#ifndef YYSTYPE_IS_DECLARED
#define YYSTYPE_IS_DECLARED 1
typedef union {
  URL_T url;
  float real;
  int   number;
  char *string;
} YYSTYPE;
#endif /* !YYSTYPE_IS_DECLARED */
#line 265 "y.tab.c"
/* compatibility with bison */
#ifdef YYPARSE_PARAM
/* compatibility with FreeBSD */
# ifdef YYPARSE_PARAM_TYPE
#  define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
# else
#  define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
# endif
#else
# define YYPARSE_DECL() yyparse(void)
#endif

/* Parameters sent to lex. */
#ifdef YYLEX_PARAM
# define YYLEX_DECL() yylex(void *YYLEX_PARAM)
# define YYLEX yylex(YYLEX_PARAM)
#else
# define YYLEX_DECL() yylex(void)
# define YYLEX yylex()
#endif

/* Parameters sent to yyerror. */
#define YYERROR_DECL() yyerror(const char *s)
#define YYERROR_CALL(msg) yyerror(msg)

extern int YYPARSE_DECL();

#define IF 257
#define ELSE 258
#define THEN 259
#define OR 260
#define FAILED 261
#define SET 262
#define LOGFILE 263
#define FACILITY 264
#define DAEMON 265
#define SYSLOG 266
#define MAILSERVER 267
#define HTTPD 268
#define ALLOW 269
#define ADDRESS 270
#define INIT 271
#define READONLY 272
#define CLEARTEXT 273
#define MD5HASH 274
#define SHA1HASH 275
#define CRYPT 276
#define DELAY 277
#define PEMFILE 278
#define ENABLE 279
#define DISABLE 280
#define HTTPDSSL 281
#define CLIENTPEMFILE 282
#define ALLOWSELFCERTIFICATION 283
#define IDFILE 284
#define STATEFILE 285
#define SEND 286
#define EXPECT 287
#define EXPECTBUFFER 288
#define CYCLE 289
#define COUNT 290
#define REMINDER 291
#define PIDFILE 292
#define START 293
#define STOP 294
#define PATHTOK 295
#define HOST 296
#define HOSTNAME 297
#define PORT 298
#define TYPE 299
#define UDP 300
#define TCP 301
#define TCPSSL 302
#define PROTOCOL 303
#define CONNECTION 304
#define ALERT 305
#define NOALERT 306
#define MAILFORMAT 307
#define UNIXSOCKET 308
#define SIGNATURE 309
#define TIMEOUT 310
#define RESTART 311
#define CHECKSUM 312
#define EVERY 313
#define DEFAULT 314
#define HTTP 315
#define APACHESTATUS 316
#define FTP 317
#define SMTP 318
#define POP 319
#define IMAP 320
#define CLAMAV 321
#define NNTP 322
#define NTP3 323
#define MYSQL 324
#define DNS 325
#define SSH 326
#define DWP 327
#define LDAP2 328
#define LDAP3 329
#define RDATE 330
#define RSYNC 331
#define TNS 332
#define PGSQL 333
#define POSTFIXPOLICY 334
#define SIP 335
#define LMTP 336
#define GPS 337
#define RADIUS 338
#define MEMCACHE 339
#define STRING 340
#define PATH 341
#define MAILADDR 342
#define MAILFROM 343
#define MAILREPLYTO 344
#define MAILSUBJECT 345
#define MAILBODY 346
#define SERVICENAME 347
#define STRINGNAME 348
#define NUMBER 349
#define PERCENT 350
#define LOGLIMIT 351
#define CLOSELIMIT 352
#define DNSLIMIT 353
#define KEEPALIVELIMIT 354
#define REPLYLIMIT 355
#define REQUESTLIMIT 356
#define STARTLIMIT 357
#define WAITLIMIT 358
#define GRACEFULLIMIT 359
#define CLEANUPLIMIT 360
#define REAL 361
#define CHECKPROC 362
#define CHECKFILESYS 363
#define CHECKFILE 364
#define CHECKDIR 365
#define CHECKHOST 366
#define CHECKSYSTEM 367
#define CHECKFIFO 368
#define CHECKSTATUS 369
#define CHILDREN 370
#define SYSTEM 371
#define RESOURCE 372
#define MEMORY 373
#define TOTALMEMORY 374
#define LOADAVG1 375
#define LOADAVG5 376
#define LOADAVG15 377
#define SWAP 378
#define MODE 379
#define ACTIVE 380
#define PASSIVE 381
#define MANUAL 382
#define CPU 383
#define TOTALCPU 384
#define CPUUSER 385
#define CPUSYSTEM 386
#define CPUWAIT 387
#define GROUP 388
#define REQUEST 389
#define DEPENDS 390
#define BASEDIR 391
#define SLOT 392
#define EVENTQUEUE 393
#define SECRET 394
#define HOSTHEADER 395
#define UID 396
#define GID 397
#define MMONIT 398
#define INSTANCE 399
#define USERNAME 400
#define PASSWORD 401
#define TIMESTAMP 402
#define CHANGED 403
#define SECOND 404
#define MINUTE 405
#define HOUR 406
#define DAY 407
#define SSLAUTO 408
#define SSLV2 409
#define SSLV3 410
#define TLSV1 411
#define CERTMD5 412
#define BYTE 413
#define KILOBYTE 414
#define MEGABYTE 415
#define GIGABYTE 416
#define INODE 417
#define SPACE 418
#define PERMISSION 419
#define SIZE 420
#define MATCH 421
#define NOT 422
#define IGNORE 423
#define ACTION 424
#define EXEC 425
#define UNMONITOR 426
#define ICMP 427
#define ICMPECHO 428
#define NONEXIST 429
#define EXIST 430
#define INVALID 431
#define DATA 432
#define RECOVERED 433
#define PASSED 434
#define SUCCEEDED 435
#define URL 436
#define CONTENT 437
#define PID 438
#define PPID 439
#define FSFLAG 440
#define REGISTER 441
#define CREDENTIALS 442
#define URLOBJECT 443
#define TARGET 444
#define MAXFORWARD 445
#define FIPS 446
#define GREATER 447
#define LESS 448
#define EQUAL 449
#define NOTEQUAL 450
#define YYERRCODE 256
static const short yylhs[] = {                           -1,
    0,    0,    1,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,   18,   18,   33,   33,
   33,   33,   33,   33,   33,   33,   33,   33,   33,   33,
   33,   33,   20,   20,   48,   48,   48,   48,   48,   48,
   48,   48,   48,   48,   48,   48,   48,   48,   48,   48,
   22,   22,   56,   56,   56,   56,   56,   56,   56,   56,
   56,   56,   56,   56,   56,   56,   56,   24,   24,   60,
   60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
   60,   60,   26,   26,   61,   61,   61,   61,   61,   61,
   61,   61,   61,   61,   61,   28,   28,   63,   63,   63,
   63,   63,   63,   63,   63,   30,   30,   65,   65,   65,
   65,   65,   65,   65,   65,   65,   65,   65,   65,   65,
   32,   32,   66,   66,   66,   66,   66,   66,    3,    3,
    3,    4,   71,   71,   14,   15,   16,    5,    5,    5,
    6,    6,    6,   12,   13,   11,    7,   73,   73,   74,
   75,   75,    8,    9,   10,   79,   79,   83,   83,   82,
   82,   86,   86,   86,   86,   87,   87,   92,   92,   92,
   91,   91,   91,   91,   91,   88,   88,   96,   96,   97,
   97,   89,   93,   94,   94,   95,   95,   90,   90,   90,
   90,   90,   90,  100,   90,  101,   90,  102,   90,  103,
   90,   90,   99,   99,  104,   98,   98,   17,   17,   17,
   17,   19,   21,   23,   25,   27,   29,   31,   34,   34,
   35,   35,  105,  105,  107,  107,  108,  108,  109,  109,
  109,  109,   84,   84,   84,   85,   85,   80,   80,   39,
   39,   40,   62,  110,  110,  111,  118,  112,  112,  112,
  112,   78,   78,   77,   77,   77,   77,   77,  113,  113,
  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
  113,  113,  113,  113,  113,  113,  113,  113,  113,  113,
  113,  113,  113,  113,  113,  113,  126,  126,  127,  127,
  123,  123,  123,  124,  124,  122,  122,  122,  128,  128,
  125,  121,  121,  129,  129,  129,  129,  129,  129,  129,
  129,  129,  129,   36,   37,   38,  120,  120,  106,  106,
   76,   76,   41,   41,  117,  117,  131,  131,   42,   42,
   42,   42,   67,  132,   68,   68,  133,  133,  133,  133,
  133,  133,  133,  133,  133,  133,  133,  133,  133,  133,
  133,  133,  133,  133,  133,  133,   69,   69,   81,   81,
  134,  134,  134,  134,   43,   44,   44,   44,   45,   46,
  135,  135,  136,   47,  137,  137,  138,  138,  138,  138,
   64,  143,  143,  144,  144,  144,  144,  139,  139,  146,
  147,  147,  147,  140,  140,  140,  140,  145,  145,  141,
  142,  149,  149,  149,  148,  148,   49,   49,  130,  130,
  130,  130,  130,  130,  150,  150,  150,  150,  150,  151,
  151,  151,  151,  151,  151,  151,  115,  152,  114,  114,
  114,  153,  153,  153,  116,  116,  116,  116,   53,   53,
   53,  154,  154,  154,   57,   57,   58,   58,   59,   72,
   72,   72,   72,   72,   50,   55,   55,   55,   55,  155,
  155,   54,   54,   51,   51,   52,   52,  119,   70,   70,
   70,
};
static const short yylen[] = {                            2,
    0,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
    2,    2,    2,    2,    2,    2,    0,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    0,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    0,    2,    1,    1,    1,
    1,    1,    1,    1,    1,    0,    2,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    7,    4,
    8,    4,    0,    3,    4,    2,    2,    3,    3,    5,
    4,    6,    4,    3,    3,    3,    3,    2,    3,    4,
    0,    2,    5,    5,    5,    1,    2,    5,    7,    0,
    2,    1,    1,    1,    1,    1,    4,    1,    2,    2,
    2,    2,    3,    3,    2,    1,    1,    2,    2,    2,
    2,    2,    2,    0,    2,    0,    1,    5,    4,    2,
    3,    3,    3,    0,    4,    0,    5,    0,    5,    0,
    5,    2,    1,    2,    1,    0,    1,    4,    4,    4,
    4,    4,    4,    4,    4,    2,    4,    4,    3,    4,
    3,    4,    1,    2,    1,    2,    1,    1,    2,    2,
    2,    2,    0,    2,    2,    0,    2,    0,    2,   11,
   10,   10,   10,    0,    2,    2,    2,    0,    2,    4,
    2,    0,    2,    0,    1,    1,    1,    1,    0,    3,
    2,    2,    2,    2,    3,    2,    2,    2,    2,    2,
    4,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    3,    2,    1,    1,    2,    2,    2,
    0,    2,    2,    0,    2,    0,    3,    5,    0,    2,
    2,    1,    3,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    7,    6,    6,    0,    2,    0,    3,
    0,    3,    7,    7,    0,    3,    1,    1,    6,    3,
    7,    1,    2,    2,    1,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    0,    4,    1,    2,
    1,    1,    1,    1,    3,    2,    2,    2,    2,    2,
    1,    2,    1,    6,    1,    2,    1,    1,    1,    1,
    6,    1,    2,    1,    1,    1,    1,    4,    4,    4,
    1,    1,    1,    4,    4,    4,    4,    4,    4,    3,
    3,    1,    1,    1,    1,    1,    9,    6,    0,    1,
    1,    1,    1,    1,    0,    1,    1,    1,    1,    1,
    2,    3,    1,    1,    1,    1,    1,    1,    0,    2,
    3,    0,    2,    3,    0,    6,    6,    6,    8,   10,
    7,    0,    1,    1,    8,    9,    9,    9,    6,    0,
    1,    1,    1,    1,    8,    7,    7,    4,    4,    0,
    1,    9,    6,    8,    8,    8,    8,    2,    0,    2,
    3,
};
static const short yydefred[] = {                         0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    3,    5,    6,    7,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   18,   27,   43,   61,   78,
    0,  106,  116,  131,    0,    0,    0,    0,  146,    0,
    0,    0,    0,    0,    0,    0,    0,  147,    0,    0,
    0,    0,    0,    0,  226,    0,    0,    4,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   95,   96,   97,   98,  100,  101,  102,  103,  104,
  105,   93,   99,    0,  342,    0,    0,    0,    0,  148,
    0,    0,    0,  166,    0,  154,  155,    0,  156,  343,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   28,   29,   30,   31,   32,   33,   34,   35,   36,   37,
   38,   39,   40,   41,   42,    0,    0,   45,   46,   47,
   49,   51,   50,   58,   59,   60,   44,   48,   52,   53,
   54,   55,   56,   57,    0,   63,   64,   65,   66,   68,
   67,   72,   73,   74,   69,   70,   71,   62,   75,   76,
   77,    0,   80,   81,   82,   84,   86,   85,   90,   91,
   92,   83,   87,   88,   89,   79,    0,    0,    0,  237,
  238,    0,  233,    0,  344,    0,  376,  377,  378,  379,
  383,    0,  381,   94,    0,    0,    0,    0,  108,  109,
  110,  111,  112,  113,  114,  107,  115,  118,  119,  120,
  122,  124,  123,  128,  129,  130,  121,  125,  126,  127,
  117,    0,  134,  133,  135,  136,  137,  138,  132,    0,
    0,  142,    0,    0,    0,    0,    0,  167,  170,  461,
  462,  463,  464,  145,  371,  372,  373,  374,    0,  369,
    0,  153,    0,    0,    0,  158,    0,    0,  349,  364,
  348,  362,  366,  354,  356,  365,  359,  363,  347,  352,
  355,  358,  357,  351,  350,  360,  361,  353,    0,  345,
    0,  140,  218,  219,  220,  221,  223,  222,  224,  225,
  227,  228,    0,    0,    0,    0,  412,  413,  414,    0,
    0,    0,    0,  385,  387,  388,  389,  390,    0,    0,
    0,    0,    0,    0,    0,  471,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  229,    0,  234,  235,  231,    0,  375,  382,    0,
    0,  340,    0,  401,  402,  403,  395,  394,    0,  392,
  396,  397,    0,  150,    0,    0,  245,  244,    0,    0,
    0,    0,  163,    0,  164,  370,    0,  268,  265,  266,
  267,    0,  159,  162,    0,    0,    0,  346,    0,    0,
    0,  424,  420,  421,  422,  423,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  386,    0,  453,  454,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  255,    0,    0,    0,    0,    0,    0,
    0,    0,  239,  241,  240,  242,  230,  236,  232,    0,
    0,    0,    0,  393,    0,  144,    0,  247,    0,  332,
  249,    0,    0,    0,    0,    0,    0,  171,  172,  173,
  174,  175,    0,    0,  186,  187,  152,    0,  160,  368,
    0,    0,  481,  257,    0,    0,  410,    0,  415,    0,
    0,    0,    0,    0,    0,    0,  440,    0,    0,  416,
  411,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  469,  468,    0,    0,    0,    0,
  478,    0,    0,    0,    0,  256,    0,    0,    0,  330,
    0,    0,    0,    0,    0,    0,    0,  168,    0,    0,
    0,    0,    0,    0,  192,  180,  189,  182,  191,  179,
  181,  188,  190,    0,    0,  185,    0,    0,  263,    0,
  139,  261,  259,    0,    0,    0,    0,    0,    0,  297,
  405,  404,  407,  406,  398,  399,    0,    0,  441,  434,
  435,  430,  433,    0,  436,    0,  437,    0,    0,    0,
    0,    0,    0,    0,  426,  427,  428,  429,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  328,    0,  337,  338,    0,    0,    0,    0,    0,    0,
  339,  409,  408,    0,  400,    0,    0,    0,    0,    0,
    0,    0,  183,  184,  193,    0,    0,  141,    0,  299,
  300,  271,    0,    0,  274,  286,  285,  276,  277,  282,
  283,  280,  272,  287,  273,  278,  279,  288,  289,  290,
  291,  284,    0,  292,  293,    0,  295,    0,  298,  325,
  326,    0,    0,  384,    0,    0,    0,    0,    0,    0,
    0,    0,  418,  473,    0,    0,    0,    0,  459,    0,
    0,    0,    0,    0,  336,    0,    0,  334,  333,  324,
  341,  391,  169,    0,    0,    0,    0,  215,    0,  213,
  217,  199,  195,  197,  177,  260,    0,  275,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  312,
    0,    0,    0,  294,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  451,    0,  467,  466,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  198,
  214,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  303,  302,    0,  281,  311,    0,    0,
    0,    0,  474,  475,  476,  477,  465,    0,  449,    0,
    0,    0,  455,    0,    0,    0,    0,    0,    0,    0,
  307,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  313,  305,    0,    0,    0,    0,    0,    0,  417,
  472,  456,  458,  457,  253,  251,    0,    0,  310,  314,
  315,  316,  317,  318,  319,  320,  321,  322,  323,  252,
  443,    0,    0,    0,    0,  450,  250,  308,  444,  438,
  446,  447,  448,
};
static const short yydgoto[] = {                         10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,   24,   25,   26,   27,   59,   28,   60,
   29,   61,   30,   62,   31,   71,   32,   86,   33,   87,
   34,   88,  121,   72,   73,   74,  125,  126,   75,  128,
   76,   77,   78,   79,   80,   81,  135,  147,  148,  149,
  150,  151,  152,  153,  154,  168,  169,  170,  171,  186,
   82,   83,  216,  217,  231,  239,   84,  289,  110,  292,
  242,  254,  105,  106,  266,  247,  382,  469,   93,  373,
  259,  374,   94,  245,  370,  458,  459,  460,  461,  462,
  463,  464,  548,  627,  705,  465,  466,  702,  699,  621,
  694,  695,  696,  700,  192,  342,  343,  193,  345,  336,
  429,  476,  558,  405,  576,  664,  515,  391,  426,  513,
  719,  708,  722,  767,  724,  559,  560,  791,  720,  397,
  605,   85,  290,  260,  202,  203,  313,  314,  315,  316,
  317,  318,  359,  360,  361,  362,  363,  480,  319,  589,
  577,  841,  806,  413,  325,
};
static const short yysindex[] = {                        -7,
 -185, -249, -224, -210, -202, -188, -145, -127, -106,    0,
   -7,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  444,    0,    0,    0, -246, -291, -279, -155,    0, -177,
 -167,  -90,  -56,  -52,  180, -286, -136,    0, -116, -183,
   20,   58,   71,  134,    0,  116,  124,    0, 1454, -165,
 1602, 1736, -238,  -75,  -75,   -1,   84,   46,   73,  103,
  444,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, -114,    0,  315, 1736,   32,  197,    0,
  174, -269, -271,    0,  130,    0,    0,  151,    0,    0,
  287,  140,  135,  176, -136,   50,  370,  372, 1664,  241,
  209,  211,   67,  213,  220,  230,  202,  248,  253, -223,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0, -217,  179,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -215,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -281,  288,  204,    0,
    0,  190,    0,  190,    0,  314,    0,    0,    0,    0,
    0,  103,    0,    0,  516, 1664,  241, 1354,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, -289,    0,    0,    0,    0,    0,    0,    0,  305,
  391,    0,  320,  -45,  275,  328,  381,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  -97,    0,
  321,    0,  274,   50,  264,    0,  287, 1664,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -115,    0,
  358,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -254,   79,   79,   79,    0,    0,    0,   79,
   79,   72,  359,    0,    0,    0,    0,    0,   79, -117,
   79, -204,   79,  204,  295,    0,  296, -123,  280,   79,
   79,  316,  382,  426,  283,  430,  390,  395,  397, -110,
   -5,    0,  -46,    0,    0,    0,  -46,    0,    0, 1664,
 1390,    0,   79,    0,    0,    0,    0,    0, 1670,    0,
    0,    0,   79,    0,  398,  330,    0,    0,  408,  274,
  347,  415,    0,  289,    0,    0,  407,    0,    0,    0,
    0,  346,    0,    0,  -92, 1492,  459,    0,  500,  449,
  499,    0,    0,    0,    0,    0,  450, -297, -277,  451,
  472,  395,  395, -262,  537,    0, -222,    0,    0,   14,
  109,  473,  515,  476,  395,  395,  517,  479,  183,  206,
  395,  482, -157,    0,  412,  543,  399,  504,  499,  559,
  595,  445,    0,    0,    0,    0,    0,    0,    0, 1546,
  459,  -93,  596,    0,  507,    0,  275,    0,  346,    0,
    0,  -47,  527, -228, -160,  303,  326,    0,    0,    0,
    0,    0,  216,  579,    0,    0,    0,  528,    0,    0,
  459,  241,    0,    0,  201,   61,    0,  534,    0,  151,
  535,  151,  536,  538,  624,  630,    0,  601,  -83,    0,
    0,  395,  395,  395,  395,  395, -252,  357,  646,  647,
  395,  151,  395,  395,    0,    0,  657,  261,  571,  151,
    0,  573,  176,  177,  176,    0,   61,  664,  -83,    0,
  459,  241,  575,  151,  -83,  576,  274,    0,  586,  588,
  589,  870,    0,  591,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  597,  598,    0,  599,  650,    0,  241,
    0,    0,    0,  274,  608,  609, 1848,  176,  336,    0,
    0,    0,    0,    0,    0,    0,  -83,  -83,    0,    0,
    0,    0,    0,  -75,    0,  678,    0,  701,  702,  703,
  704,  705,  626,  708,    0,    0,    0,    0,  395,  -83,
  -83,  718,  395,  719,  720,  -83,  395,  721,  395,  395,
    0,  395,    0,    0,  645,  395,  176, -192,  678,  241,
    0,    0,    0,  678,    0,  346,    0,    0,    0,  648,
  651,  709,    0,    0,    0,  649,  710,    0,  346,    0,
    0,    0,  603, 1248,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  542,    0,    0,  602,    0,  395,    0,    0,
    0,  -64,  744,    0,  -83,  -83,  -83,  -83,  -83,  395,
  -83,  763,    0,    0,  -83,  764,  -83,  -83,    0,  770,
  -83,  771,  772,  777,    0,  778,  395,    0,    0,    0,
    0,    0,    0,  651,  651,  651,  709,    0,  651,    0,
    0,    0,    0,    0,    0,    0,  700,    0,   79,   79,
   79,   79,   79,   79,   79,   79,   79,   79,  782,    0,
   -6,  605,  707,    0,  784,  246,   86,  678,  678,  678,
  678,  678,  790,  678,  -83,    0,  -83,    0,    0,  -83,
  678,  -83,  -83,  -83,  -83,  800,  651,  651,  651,    0,
    0, -253,  711,  712,  713,  715,  716,  723,  724,  726,
  734,  736, 1248,    0,    0,  738,    0,    0,  -83,  739,
  739,  739,    0,    0,    0,    0,    0,  -83,    0,  678,
  678,  678,    0,  678,  678,  678,  678,  -83,  729,  757,
    0,  749,  750,  751,  752,  753,  756,  759,  766,  768,
  774,    0,    0,  678, -234,  807,  808,  817,  678,    0,
    0,    0,    0,    0,    0,    0,  678,  725,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  818,  -83,  -83,  -83,    0,    0,    0,    0,    0,
    0,    0,    0,
};
static const short yyrindex[] = {                      1113,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1114,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  497,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  513,  641,
  824,  937,    0,    0,    0,    0,    0,    0,    0,    0,
 1048,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  172,    0, 1108, 1171, 1197, 1216,    0,
 1228,  533,  590,    0,    0,    0,    0, 1247,    0,    0,
    0,    0,    0,    5, 1258,   30,    0,    0,    0, 1269,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  714,  714,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  827,    0,    0,    0,
    0,  291,    0,  291,    0,    0,    0,    0,    0,    0,
    0,  331,    0,    0,    0,    0,  418,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  607,    0, 1285,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1293,    0,   13,   30,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  827,  788,  -63,  -63,    0,    0,    0,  788,
  788,    0,  867,    0,    0,    0,    0,    0,  -63,  826,
  788,  826,  788,  728,    0,    0,    0,    0,    0,  788,
  -63,    0,    0,    0,    0,    0,    0,  867,    0,    0,
    0,    0,  291,    0,    0,    0,  291,    0,    0,    0,
    0,    0,  -63,    0,    0,    0,    0,    0,  867,    0,
    0,    0,  788,    0,    0,  533,    0,    0,    0,  405,
    0,    0,    0, 1311,    0,    0,    0,    0,    0,    0,
    0,   21,    0,    0,    0,    0,  497,    0,  447,    0,
  165,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  867,  867,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  867,  867,    0,    0,    0,    0,
  867,    0,    0,    0,    0, -233, -225,    0,  165,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  172,    0,    0,    0,    0,    0,  607,    0, 1159,    0,
    0,    0,    0,    0,    0,  862,    0,    0,    0,    0,
    0,    0,  950,    0,    0,    0,    0,    0,    0,    0,
  497, 1269,    0,    0,    0, -134,    0, 1339,    0, 1758,
 1339, 1758,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  867,  867,  867,  867,  867,  867, -235,    0,    0,
  867, -219,  867,  867,    0,    0,    0,  867,  -86, -219,
    0,    0, -211,    0, -211,    0, -134,    0,    0,    0,
  172,  418, 1638, 1774,    0,    0,  405,    0,    0,    0,
    0,  965,  765,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  801,    0, 1269,
    0,    0,    0, -193,    0,    0,    0, -211,  -50,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  689,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  867,    0,
    0,    0,  867,    0,    0,    0,  867,    0,  867,  867,
    0,  867,    0,    0,    0,  867, -211,    0,  545,  418,
    0,    0,    0,  727,    0, 1159,  842,  874,  915,    0,
    0,  987,    0,    0,    0,    0, 1028,    0,  173,    0,
    0,    0,  -38,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -245,    0,    0,    0,    0,  867,    0,    0,
    0,  212,    0,    0,    0,    0,    0,    0,    0,  867,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  867,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  987,    0, 1078,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  788,  788,
  788,  788,  788,  788,  788,  788,  788,  788,    3,    0,
    0,  143,    0,    0,    0,  250,    0,  545,  545,  545,
  545,  545,    0,  545,    0,    0,    0,    0,    0,    0,
  689,    0,    0,    0,    0,    0, 1098, 1119, 1139,    0,
    0,  147,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  882,
  882,  882,    0,    0,    0,    0,    0,    0,    0,  545,
  545,  689,    0,  689,  689,  689,  689,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  689,    0,    0,    0,    0,  545,    0,
    0,    0,    0,    0,    0,    0,  689,  147,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,
};
static const short yygindex[] = {                         0,
    0, 1131,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  785,  952,  428,    0,    0, 1086,    0,
  604,  717,  732,  587,  832,  938,    0,    0,  -11,    1,
  239,  278,    0,    0,    0,    0,    0,    0,    0,    0,
 1075,    0,    0,    0,    0,    0, 1146, -132,  -41, -201,
    0,  -79,    0, 1043,  886, -103, -348, -433,    0,    0,
  885,    0, 1063,  791,  731,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  461,  -44,    0,
    0,    0,    0, -429,  -65, -153, -191, -190, -335,    0,
    0,  733,  643, 1280, -500, 1223,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  613,  343,  410, -274,
    0,    0, -216, -241,    0,  972,    0,  863,    0, -197,
    0, -158,    0,  816,    0,    0,    0,  150,    0,    0,
 -147, -173, -100,  857, 1044,
};
#define YYTABLESIZE 2187
static const short yytable[] = {                        194,
  263,  344,  347,  344,  331,  352,  109,  438,  206,  387,
  357,  438,  264,  301,  333,  528,  534,  376,  609,   89,
  262,  449,  187,  425,  614,  327,  487,  375,  243,  161,
  398,  399,  470,  335,  583,  400,  401,  303,  246,  460,
  346,  333,  207,  320,  407,  328,  414,  331,  418,  358,
  182,  478,  536,  390,  831,  422,  423,   91,  789,  188,
   92,  165,  183,  479,  301,  264,  660,  661,   92,  408,
  409,  481,  388,  351,  328,  227,  327,   35,  442,   36,
  537,   37,   38,  479,  335,   39,  488,  228,  445,  673,
  674,  136,  264,  264,   90,  679,  404,   50,   40,   41,
  570,  571,   42,  301,  102,  103,   43,  689,  111,  264,
  188,  112,  572,  425,  832,  327,  264,  688,  573,   44,
  538,   45,   51,  335,  269,  188,  490,   64,   65,  460,
  244,  188,  189,  188,  388,  386,   52,  331,  479,   44,
   66,  790,   95,  376,   53,  334,  304,   67,  539,  305,
  306,  307,  308,  309,  335,  264,  408,  409,   54,  310,
  311,  357,  188,   96,  728,  729,  730,  731,  732,  388,
  734,  367,  416,   97,  736,  269,  738,  739,  616,  312,
  741,  335,  693,  189,  321,  322,  321,  332,  269,  437,
  107,  509,  107,  439,  270,  706,  271,  415,  189,  301,
  358,   55,  323,  479,  324,  629,  189,   46,  296,  570,
  571,  431,   47,   68,  269,  416,  329,  440,  264,   56,
  306,  572,   69,  388,   70,  529,  530,  573,  531,  433,
  330,  331,  574,  575,  780,  189,  781,  113,  434,  782,
   57,  784,  785,  786,  787,  255,  256,  257,  258,  432,
  255,  256,  257,  258,    1,  523,  272,  137,   98,  296,
   48,  270,  416,  339,  190,  191,  331,  479,  804,  751,
  551,  306,  410,  411,  264,  190,  191,  809,  410,  411,
  273,  274,  262,  275,   99,  419,  276,  817,  232,  100,
  329,  161,  532,  533,  367,  412,  368,  419,  296,  166,
  184,  412,  101,  277,  278,  108,  104,  205,  279,  280,
  306,  281,  270,  282,  114,  283,  284,  751,  751,  751,
  611,  285,  286,  287,  288,  229,  416,  416,  416,  416,
  380,  340,  341,  764,  435,  765,   44,   66,  167,  185,
  195,  574,  575,  436,   67,  472,  555,  556,  628,  340,
  341,  270,  115,  492,    2,    3,    4,    5,    6,    7,
    8,    9,  493,  557,  230,  116,  331,  331,  331,  331,
  331,  331,  331,  331,  264,  264,  264,  264,  264,  264,
  264,  264,  262,  262,  262,  262,  262,  262,  262,  262,
  438,  161,  161,  161,  161,  161,  161,  161,  161,  522,
  562,  304,  564,  117,  264,  309,  295,  296,  691,  602,
  118,  606,  331,  331,  331,  331,  331,  479,  119,   69,
  200,   70,  593,  258,  264,  197,  198,  199,  367,  550,
  600,  262,  196,  367,  753,  754,  755,  756,  757,  758,
  759,  760,  761,  762,  613,  331,  480,  331,  494,  201,
  258,  258,  304,  264,  658,  264,  309,  495,  262,  262,
  240,  262,  367,  262,  367,  367,  241,  258,  431,  431,
  726,  344,  161,  431,  258,  262,  367,  367,  249,  610,
  261,  392,  262,  262,  367,  246,  124,  140,  158,  175,
  265,  304,  267,  544,  268,  309,  367,  545,  546,  339,
  552,  553,  554,  687,  431,  431,  432,  432,  662,  402,
  403,  432,   19,  258,  220,  233,  431,  431,  770,  771,
  772,  262,  503,  504,  431,  393,  394,  395,  396,  190,
  191,  291,  243,  367,  367,  367,  367,  367,  367,  367,
  367,  300,  432,  432,  445,  505,  506,  329,  482,  293,
  367,  294,  329,  297,  432,  432,  491,  452,  453,  367,
  298,  367,  432,  250,  251,  252,  253,  454,  455,  456,
  299,  208,  510,  431,  431,  431,  431,  431,  431,  431,
  431,  540,  541,  329,  329,  340,  341,  380,  301,  331,
  431,  524,  380,  302,  367,  329,  329,  457,  337,  431,
  326,  431,  348,  329,  542,  543,  246,   64,   65,  404,
  597,  432,  432,  432,  432,  432,  432,  432,  432,   44,
   66,  555,  556,  380,  380,  603,  604,   67,  432,  255,
  256,  257,  258,  338,  431,  380,  380,  432,  350,  432,
   20,  340,  341,  380,  364,  132,  144,  162,  179,  747,
  748,  749,  329,  329,  329,  329,  329,  329,  329,  329,
  842,  843,  129,  141,  159,  176,  264,  365,  366,  329,
  807,  808,  432,  224,  479,  369,  371,  372,  329,  479,
  329,  378,  379,  380,  381,  840,  840,  840,  445,  211,
  221,  234,  380,  380,  380,  380,  380,  380,  380,  380,
   63,  264,   69,  480,   70,  384,  389,  404,  480,  380,
  479,  479,  377,  329,  264,  419,  420,  415,  380,  421,
  380,  424,  479,  479,  425,  427,  445,  428,  304,  244,
  479,  305,  306,  307,  308,  309,   64,   65,  430,  480,
  480,  310,  311,  404,  264,  432,  446,  448,   44,   66,
  450,  480,  480,  380,  451,  467,   67,  468,  367,  480,
  585,  586,  587,  588,  200,  107,  264,  264,  264,  264,
  264,  264,  264,  264,   19,  130,  142,  160,  177,  479,
  479,  479,  479,  479,  479,  479,  479,  367,  473,  474,
  131,  143,  161,  178,  243,  489,  479,  475,  477,  483,
  194,  445,  212,  222,  235,  479,  445,  479,  480,  480,
  480,  480,  480,  480,  480,  480,  264,  213,  223,  236,
  484,  496,   68,   21,  498,  480,  497,  502,  501,  243,
  508,   69,  512,   70,  480,  514,  480,  445,  445,  511,
  479,  201,  243,  122,  138,  156,  173,  518,  520,  445,
  445,  331,  516,  519,  525,  526,  547,  445,  367,  367,
  367,  367,  367,  367,  367,  367,  535,  549,  246,  480,
  209,  218,  243,  202,   19,   19,   19,   19,   19,   19,
   19,   19,  567,  561,  563,  565,  331,  566,  568,  569,
  133,  145,  163,  180,  243,  243,  243,  243,  243,  243,
  243,  243,   20,  246,  590,  591,  445,  445,  445,  445,
  445,  445,  445,  445,  203,  596,  246,  214,  225,  237,
  599,  601,  608,  445,  612,  615,  617,  620,  618,  619,
  622,  626,  445,  243,  445,  663,   22,  623,  624,  625,
  243,  243,  243,  243,  243,  445,  246,  630,  631,  176,
  445,  331,  331,  331,  331,  331,  331,  331,  331,  665,
  666,  667,  668,  669,  212,  670,  671,  445,  246,  246,
  246,  246,  246,  246,  246,  246,  675,  677,  678,  681,
  701,  445,  445,  445,  685,  721,  216,  697,  445,  703,
  698,  707,  704,  445,  445,  723,  134,  146,  164,  181,
  727,  445,   20,   20,   20,   20,   20,   20,   20,   20,
  123,  139,  157,  174,  246,  246,  246,  246,  246,  445,
  445,  735,  737,  215,  226,  238,  200,  196,  740,  742,
  743,  445,  445,  200,  200,  744,  745,  210,  219,  445,
  752,  763,  769,  200,  200,  200,  768,   23,  778,  766,
  445,  445,  445,  445,  445,  445,  445,  445,  788,  792,
  793,  794,  194,  795,  796,  833,  834,  445,  818,  194,
  194,  797,  798,  200,  799,  835,  445,  205,  445,  194,
  194,  194,  800,  194,  801,   21,  803,  805,  445,  445,
  445,  445,  445,  445,  445,  445,  819,  207,  820,  821,
  822,  823,  824,  201,  204,  825,  839,   24,  826,  194,
  201,  201,    1,    2,  445,  827,  445,  828,  209,  790,
  201,  201,  201,  829,  254,  439,  200,  200,  200,  200,
  200,  200,  200,  200,  470,  202,  419,  452,  211,  178,
  442,   58,  202,  202,  127,  204,   49,  264,  471,  383,
  201,  385,  202,  202,  202,  248,  447,  750,  262,  607,
  838,  517,  194,  194,  194,  194,  194,  194,  194,  194,
   25,  659,  802,  349,  444,  406,  203,  527,  417,    0,
  327,  206,  202,  203,  203,   21,   21,   21,   21,   21,
   21,   21,   21,  203,  203,  203,   26,    0,   22,    0,
    0,    0,    0,  201,  201,  201,  201,  201,  201,  201,
  201,  176,    0,  208,    0,  149,    0,    0,  176,  176,
    0,    0,    0,  203,    0,    0,  212,  143,  176,  176,
  176,    0,    0,  212,  212,  202,  202,  202,  202,  202,
  202,  202,  202,  212,  212,  212,  460,    0,  216,    0,
    0,    0,    0,    0,  210,  216,  216,  157,  176,    0,
    0,    0,    0,    0,    0,  216,  216,  216,  479,    0,
    0,    0,    0,  212,    0,    0,  203,  203,  203,  203,
  203,  203,  203,  203,  248,    0,    0,    0,    0,  196,
    0,    0,  151,    0,    0,  216,  196,  196,   22,   22,
   22,   22,   22,   22,   22,   22,  196,  196,  196,   23,
  165,  176,  176,  176,  176,  176,  176,  176,  176,    0,
    0,    0,    0,    0,    0,    0,  212,  212,  212,  212,
  212,  212,  212,  212,    0,    0,  196,    0,    0,  205,
    0,    0,    0,    0,    0,    0,  205,  205,  216,  216,
  216,  216,  216,  216,  216,  216,  205,  205,  205,  207,
    0,    0,    0,    0,    0,    0,  207,  207,    0,   24,
    0,    0,    0,    0,    0,    0,  207,  207,  207,    0,
  209,    0,    0,    0,    0,    0,  205,  209,  209,  196,
  196,  196,  196,  196,  196,  196,  196,  209,  209,  209,
  211,    0,    0,    0,    0,    0,  207,  211,  211,   23,
   23,   23,   23,   23,   23,   23,   23,  211,  211,  211,
  262,    0,    0,    0,    0,    0,    0,  209,    0,    0,
    0,    0,   25,    0,    0,    0,    0,    0,    0,  205,
  205,  205,  205,  205,  205,  205,  205,  211,    0,    0,
    0,    0,    0,    0,    0,  262,    0,    0,   26,  207,
  207,  207,  207,  207,  207,  207,  207,    0,  262,   24,
   24,   24,   24,   24,   24,   24,   24,  149,    0,    0,
  209,  209,  209,  209,  209,  209,  209,  209,    0,  143,
    0,    0,    0,    0,    0,    0,    0,    0,  262,    0,
  211,  211,  211,  211,  211,  211,  211,  211,  460,    0,
    0,    0,    0,    0,  441,    0,    0,    0,    0,  157,
  262,  262,  262,  262,  262,  262,  262,  262,    0,    0,
  479,    0,   25,   25,   25,   25,   25,   25,   25,   25,
    0,    0,    0,    0,    0,    0,  248,    0,    0,    0,
    0,    0,    0,    0,  151,    0,    0,    0,   26,   26,
   26,   26,   26,   26,   26,   26,    0,    0,    0,    0,
    0,    0,  165,    0,    0,    0,    0,  149,  149,  149,
  149,  149,  149,  149,  149,    0,    0,    0,    0,  143,
  143,  143,  143,  143,  143,  143,  143,  416,  709,  710,
  711,  712,  713,  714,  715,  716,  717,  718,  460,  460,
  460,  460,  460,  460,  460,  460,  471,  431,    0,  157,
  157,  157,  157,  157,  157,  157,  157,    0,    0,    0,
  479,  479,  479,  479,  479,  479,  479,  479,  443,    0,
    0,    0,    0,    0,    0,    0,  248,  248,  248,  248,
  248,  248,  248,  248,  151,  151,  151,  151,  151,  151,
  151,  151,    0,    0,    0,    0,    0,    0,    0,    0,
  521,    0,  165,  165,  165,  165,  165,  165,  165,  165,
    0,  485,  486,    0,    0,    0,    0,  416,    0,    0,
    0,    0,    0,  269,  499,  500,    0,    0,    0,  270,
  507,  271,  188,    0,    0,    0,    0,    0,  416,    0,
  120,  416,  416,  416,  416,  416,  416,    0,    0,    0,
    0,  416,  416,  416,  416,  416,  305,  306,  307,  308,
  309,  353,    0,    0,    0,    0,    0,    0,  354,  355,
  356,    0,    0,    0,    0,    0,   64,   65,    0,    0,
    0,  416,  416,  416,  416,    0,    0,    0,   44,   66,
    0,  272,    0,    0,    0,    0,   67,    0,    0,    0,
    0,  578,  579,  580,  581,  582,  584,    0,    0,    0,
  592,    0,  594,  595,    0,  273,  274,  598,  275,    0,
    0,  276,    0,    0,    0,  269,    0,    0,    0,    0,
    0,  270,    0,  271,    0,    0,    0,    0,  277,  278,
    0,    0,    0,  279,  280,    0,  281,    0,  282,    0,
  283,  284,    0,    0,    0,    0,  285,  286,  287,  288,
    0,  690,   68,    0,    0,    0,  692,    0,    0,    0,
    0,   69,    0,   70,    0,    0,    0,    0,    0,  269,
    0,    0,    0,    0,    0,  270,    0,  271,  155,    0,
    0,    0,    0,  272,    0,    0,    0,    0,  672,    0,
    0,    0,  676,    0,    0,    0,  680,    0,  682,  683,
    0,  684,    0,    0,    0,  686,    0,  273,  274,    0,
  275,    0,    0,  276,   64,   65,  416,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   44,   66,    0,    0,
  277,  278,    0,    0,   67,  279,  280,  272,  281,    0,
  282,    0,  283,  284,    0,    0,    0,    0,  285,  286,
  287,  288,    0,    0,    0,    0,    0,  725,    0,    0,
    0,  273,  274,    0,  275,    0,    0,  276,    0,  733,
  773,  774,  775,  776,  777,    0,  779,    0,    0,    0,
    0,    0,    0,  783,  277,  278,  746,  269,    0,  279,
  280,    0,  281,  270,  282,  271,  283,  284,    0,    0,
   68,    0,  285,  286,  287,  288,  416,    0,    0,   69,
    0,   70,  172,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  810,  811,  812,    0,  813,  814,  815,  816,
  416,  416,  416,  416,  416,  416,  460,    0,  404,    0,
    0,    0,  416,  416,  416,    0,  830,    0,   64,   65,
    0,  836,  460,    0,    0,  272,    0,    0,    0,  837,
   44,   66,  305,  306,  307,  308,  309,  353,   67,    0,
  416,  416,  416,  416,  354,  355,  356,    0,    0,  273,
  274,    0,  275,    0,    0,  276,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  277,  278,    0,    0,    0,  279,  280,    0,
  281,    0,  282,    0,  283,  284,    0,    0,    0,    0,
  285,  286,  287,  288,    0,    0,  460,    0,    0,    0,
    0,    0,    0,    0,   68,    0,    0,    0,    0,    0,
    0,    0,  460,   69,    0,   70,    0,  460,    0,    0,
  460,  460,  460,  460,  460,  460,    0,    0,    0,    0,
  460,  460,  460,  460,  460,    0,  460,  460,  460,  460,
  460,  460,    0,    0,    0,    0,    0,    0,  460,  460,
  460,  632,  633,  634,  635,  636,  637,  638,  639,  640,
  641,  642,  643,  644,  645,  646,  647,  648,  649,  650,
  651,  652,  653,  654,  655,  656,  657,
};
static const short yycheck[] = {                         65,
  104,  192,  194,  194,    0,  207,  123,  343,  123,  125,
  208,  347,    0,  259,  296,  449,   64,  259,  519,  266,
    0,  370,  261,  259,  525,  259,  289,  125,  298,    0,
  305,  306,  125,  259,  287,  310,  311,  261,  310,  259,
  194,  296,   84,  261,  319,  261,  321,  259,  323,  208,
   62,  349,  281,  308,  289,  330,  331,  349,  312,  349,
  340,   61,   62,  361,  310,  259,  567,  568,  340,  274,
  275,  349,  289,  206,  261,   87,  310,  263,  353,  265,
  309,  267,  268,  361,  310,  271,  349,   87,  363,  590,
  591,  257,  286,  287,  341,  596,  349,  347,  284,  285,
  293,  294,  288,  349,  391,  392,  292,  608,  292,  303,
  349,  295,  305,  349,  349,  349,  310,  310,  311,  305,
  281,  307,  347,  349,  259,  349,  349,  293,  294,  349,
  400,  349,  422,  349,  351,  268,  347,  349,  361,  305,
  306,  395,  298,  385,  347,  427,  370,  313,  309,  373,
  374,  375,  376,  377,  436,  349,  274,  275,  347,  383,
  384,  359,  349,  341,  665,  666,  667,  668,  669,  386,
  671,    0,  259,  341,  675,  310,  677,  678,  527,  403,
  681,  436,  616,  422,  402,  403,  402,  403,  304,  343,
  307,  349,  307,  347,  310,  629,  312,  402,  422,  445,
  359,  347,  420,  361,  422,  554,  422,  393,  259,  293,
  294,    0,  398,  379,  349,  420,  403,  350,  412,  347,
  259,  305,  388,  440,  390,  273,  274,  311,  276,  340,
  417,  418,  425,  426,  735,  422,  737,  421,  349,  740,
  347,  742,  743,  744,  745,  343,  344,  345,  346,    0,
  343,  344,  345,  346,  262,  349,  372,  423,  349,  310,
  446,  259,  349,  310,  340,  341,  262,  361,  769,  699,
  472,  310,  396,  397,  262,  340,  341,  778,  396,  397,
  396,  397,  262,  399,  341,  349,  402,  788,  257,  342,
    0,  262,  340,  341,  340,  419,  342,  361,  349,   61,
   62,  419,  123,  419,  420,  422,  443,  422,  424,  425,
  349,  427,  310,  429,  295,  431,  432,  747,  748,  749,
  522,  437,  438,  439,  440,   87,  413,  414,  415,  416,
    0,  396,  397,  340,  340,  342,  305,  306,   61,   62,
  342,  425,  426,  349,  313,  387,  286,  287,  550,  396,
  397,  349,  295,  340,  362,  363,  364,  365,  366,  367,
  368,  369,  349,  303,   87,  295,  362,  363,  364,  365,
  366,  367,  368,  369,  362,  363,  364,  365,  366,  367,
  368,  369,  362,  363,  364,  365,  366,  367,  368,  369,
  726,  362,  363,  364,  365,  366,  367,  368,  369,  441,
  480,  259,  482,  270,    0,  259,  340,  341,  610,  513,
  295,  515,  408,  409,  410,  411,  412,    0,  295,  388,
  348,  390,  502,  259,  412,  380,  381,  382,  257,  471,
  510,  259,  349,  262,  709,  710,  711,  712,  713,  714,
  715,  716,  717,  718,  524,  441,    0,  443,  340,  347,
  286,  287,  310,  441,  558,  443,  310,  349,  286,  287,
  264,  441,  291,  443,  293,  294,  293,  303,  257,  258,
  662,  662,  443,  262,  310,  303,  305,  306,  349,  521,
  341,  403,  310,  349,  313,  310,   59,   60,   61,   62,
  441,  349,  123,  278,  123,  349,    0,  282,  283,  310,
  300,  301,  302,  607,  293,  294,  257,  258,  574,  438,
  439,  262,    0,  349,   87,   88,  305,  306,  433,  434,
  435,  349,  340,  341,  313,  447,  448,  449,  450,  340,
  341,  291,    0,  362,  363,  364,  365,  366,  367,  368,
  369,  340,  293,  294,    0,  340,  341,  257,  399,  341,
  379,  341,  262,  341,  305,  306,  407,  269,  270,  388,
  341,  390,  313,  413,  414,  415,  416,  279,  280,  281,
  341,  257,  423,  362,  363,  364,  365,  366,  367,  368,
  369,  279,  280,  293,  294,  396,  397,  257,  341,    0,
  379,  442,  262,  341,  423,  305,  306,  309,  311,  388,
  422,  390,  289,  313,  279,  280,    0,  293,  294,  349,
  350,  362,  363,  364,  365,  366,  367,  368,  369,  305,
  306,  286,  287,  293,  294,  449,  450,  313,  379,  343,
  344,  345,  346,  430,  423,  305,  306,  388,  123,  390,
    0,  396,  397,  313,  340,   59,   60,   61,   62,  694,
  695,  696,  362,  363,  364,  365,  366,  367,  368,  369,
  834,  835,   59,   60,   61,   62,  262,  277,  349,  379,
  771,  772,  423,   87,  257,  401,  349,  297,  388,  262,
  390,  408,  409,  410,  411,  833,  834,  835,    0,   86,
   87,   88,  362,  363,  364,  365,  366,  367,  368,  369,
  257,  297,  388,  257,  390,  442,  349,  349,  262,  379,
  293,  294,  392,  423,  310,  421,  421,  402,  388,  440,
  390,  340,  305,  306,  299,  443,    0,  298,  370,  400,
  313,  373,  374,  375,  376,  377,  293,  294,  349,  293,
  294,  383,  384,  349,  340,  349,  349,  340,  305,  306,
  404,  305,  306,  423,  340,  349,  313,  412,  262,  313,
  404,  405,  406,  407,    0,  307,  362,  363,  364,  365,
  366,  367,  368,  369,  262,   59,   60,   61,   62,  362,
  363,  364,  365,  366,  367,  368,  369,  291,  289,  341,
   59,   60,   61,   62,  262,  259,  379,  299,  349,  349,
    0,  257,   86,   87,   88,  388,  262,  390,  362,  363,
  364,  365,  366,  367,  368,  369,  412,   86,   87,   88,
  349,  349,  379,    0,  349,  379,  312,  349,  312,  297,
  349,  388,  290,  390,  388,  437,  390,  293,  294,  428,
  423,    0,  310,   59,   60,   61,   62,  289,  404,  305,
  306,  262,  349,  259,  259,  349,  278,  313,  362,  363,
  364,  365,  366,  367,  368,  369,  340,  340,  262,  423,
   86,   87,  340,    0,  362,  363,  364,  365,  366,  367,
  368,  369,  259,  350,  350,  350,  297,  350,  259,  289,
   59,   60,   61,   62,  362,  363,  364,  365,  366,  367,
  368,  369,  262,  297,  259,  259,  362,  363,  364,  365,
  366,  367,  368,  369,    0,  259,  310,   86,   87,   88,
  350,  349,  259,  379,  350,  350,  341,   58,  341,  341,
  340,  282,  388,  401,  390,  258,    0,  341,  341,  341,
  408,  409,  410,  411,  412,  257,  340,  340,  340,    0,
  262,  362,  363,  364,  365,  366,  367,  368,  369,  259,
  259,  259,  259,  259,    0,  340,  259,  423,  362,  363,
  364,  365,  366,  367,  368,  369,  259,  259,  259,  259,
  272,  293,  294,  257,  340,  444,    0,  340,  262,  341,
  340,  389,  283,  305,  306,  394,   59,   60,   61,   62,
  257,  313,  362,  363,  364,  365,  366,  367,  368,  369,
   59,   60,   61,   62,  408,  409,  410,  411,  412,  293,
  294,  259,  259,   86,   87,   88,  262,    0,  259,  259,
  259,  305,  306,  269,  270,  259,  259,   86,   87,  313,
  341,  260,  259,  279,  280,  281,  340,    0,  259,  445,
  362,  363,  364,  365,  366,  367,  368,  369,  259,  349,
  349,  349,  262,  349,  349,  259,  259,  379,  340,  269,
  270,  349,  349,  309,  349,  259,  388,    0,  390,  279,
  280,  281,  349,  283,  349,  262,  349,  349,  362,  363,
  364,  365,  366,  367,  368,  369,  340,    0,  350,  350,
  350,  350,  350,  262,  340,  350,  289,    0,  350,  309,
  269,  270,    0,    0,  388,  350,  390,  350,    0,  395,
  279,  280,  281,  350,  298,  259,  362,  363,  364,  365,
  366,  367,  368,  369,  421,  262,  349,  312,    0,  278,
  259,   11,  269,  270,   59,   71,    1,  105,  421,  264,
  309,  267,  279,  280,  281,   93,  366,  697,    0,  517,
  818,  429,  362,  363,  364,  365,  366,  367,  368,  369,
    0,  559,  763,  202,  359,  313,  262,  447,  322,   -1,
  137,  340,  309,  269,  270,  362,  363,  364,  365,  366,
  367,  368,  369,  279,  280,  281,    0,   -1,  262,   -1,
   -1,   -1,   -1,  362,  363,  364,  365,  366,  367,  368,
  369,  262,   -1,  340,   -1,    0,   -1,   -1,  269,  270,
   -1,   -1,   -1,  309,   -1,   -1,  262,    0,  279,  280,
  281,   -1,   -1,  269,  270,  362,  363,  364,  365,  366,
  367,  368,  369,  279,  280,  281,    0,   -1,  262,   -1,
   -1,   -1,   -1,   -1,  340,  269,  270,    0,  309,   -1,
   -1,   -1,   -1,   -1,   -1,  279,  280,  281,    0,   -1,
   -1,   -1,   -1,  309,   -1,   -1,  362,  363,  364,  365,
  366,  367,  368,  369,    0,   -1,   -1,   -1,   -1,  262,
   -1,   -1,    0,   -1,   -1,  309,  269,  270,  362,  363,
  364,  365,  366,  367,  368,  369,  279,  280,  281,  262,
    0,  362,  363,  364,  365,  366,  367,  368,  369,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,  363,  364,  365,
  366,  367,  368,  369,   -1,   -1,  309,   -1,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,  269,  270,  362,  363,
  364,  365,  366,  367,  368,  369,  279,  280,  281,  262,
   -1,   -1,   -1,   -1,   -1,   -1,  269,  270,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,  281,   -1,
  262,   -1,   -1,   -1,   -1,   -1,  309,  269,  270,  362,
  363,  364,  365,  366,  367,  368,  369,  279,  280,  281,
  262,   -1,   -1,   -1,   -1,   -1,  309,  269,  270,  362,
  363,  364,  365,  366,  367,  368,  369,  279,  280,  281,
  262,   -1,   -1,   -1,   -1,   -1,   -1,  309,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,   -1,  362,
  363,  364,  365,  366,  367,  368,  369,  309,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  297,   -1,   -1,  262,  362,
  363,  364,  365,  366,  367,  368,  369,   -1,  310,  362,
  363,  364,  365,  366,  367,  368,  369,  262,   -1,   -1,
  362,  363,  364,  365,  366,  367,  368,  369,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  340,   -1,
  362,  363,  364,  365,  366,  367,  368,  369,  262,   -1,
   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,  262,
  362,  363,  364,  365,  366,  367,  368,  369,   -1,   -1,
  262,   -1,  362,  363,  364,  365,  366,  367,  368,  369,
   -1,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,  362,  363,
  364,  365,  366,  367,  368,  369,   -1,   -1,   -1,   -1,
   -1,   -1,  262,   -1,   -1,   -1,   -1,  362,  363,  364,
  365,  366,  367,  368,  369,   -1,   -1,   -1,   -1,  362,
  363,  364,  365,  366,  367,  368,  369,  259,  351,  352,
  353,  354,  355,  356,  357,  358,  359,  360,  362,  363,
  364,  365,  366,  367,  368,  369,  125,  338,   -1,  362,
  363,  364,  365,  366,  367,  368,  369,   -1,   -1,   -1,
  362,  363,  364,  365,  366,  367,  368,  369,  359,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  362,  363,  364,  365,
  366,  367,  368,  369,  362,  363,  364,  365,  366,  367,
  368,  369,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  125,   -1,  362,  363,  364,  365,  366,  367,  368,  369,
   -1,  402,  403,   -1,   -1,   -1,   -1,  349,   -1,   -1,
   -1,   -1,   -1,  304,  415,  416,   -1,   -1,   -1,  310,
  421,  312,  349,   -1,   -1,   -1,   -1,   -1,  370,   -1,
  257,  373,  374,  375,  376,  377,  378,   -1,   -1,   -1,
   -1,  383,  384,  385,  386,  387,  373,  374,  375,  376,
  377,  378,   -1,   -1,   -1,   -1,   -1,   -1,  385,  386,
  387,   -1,   -1,   -1,   -1,   -1,  293,  294,   -1,   -1,
   -1,  413,  414,  415,  416,   -1,   -1,   -1,  305,  306,
   -1,  372,   -1,   -1,   -1,   -1,  313,   -1,   -1,   -1,
   -1,  492,  493,  494,  495,  496,  497,   -1,   -1,   -1,
  501,   -1,  503,  504,   -1,  396,  397,  508,  399,   -1,
   -1,  402,   -1,   -1,   -1,  304,   -1,   -1,   -1,   -1,
   -1,  310,   -1,  312,   -1,   -1,   -1,   -1,  419,  420,
   -1,   -1,   -1,  424,  425,   -1,  427,   -1,  429,   -1,
  431,  432,   -1,   -1,   -1,   -1,  437,  438,  439,  440,
   -1,  609,  379,   -1,   -1,   -1,  614,   -1,   -1,   -1,
   -1,  388,   -1,  390,   -1,   -1,   -1,   -1,   -1,  304,
   -1,   -1,   -1,   -1,   -1,  310,   -1,  312,  257,   -1,
   -1,   -1,   -1,  372,   -1,   -1,   -1,   -1,  589,   -1,
   -1,   -1,  593,   -1,   -1,   -1,  597,   -1,  599,  600,
   -1,  602,   -1,   -1,   -1,  606,   -1,  396,  397,   -1,
  399,   -1,   -1,  402,  293,  294,  259,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  305,  306,   -1,   -1,
  419,  420,   -1,   -1,  313,  424,  425,  372,  427,   -1,
  429,   -1,  431,  432,   -1,   -1,   -1,   -1,  437,  438,
  439,  440,   -1,   -1,   -1,   -1,   -1,  658,   -1,   -1,
   -1,  396,  397,   -1,  399,   -1,   -1,  402,   -1,  670,
  728,  729,  730,  731,  732,   -1,  734,   -1,   -1,   -1,
   -1,   -1,   -1,  741,  419,  420,  687,  304,   -1,  424,
  425,   -1,  427,  310,  429,  312,  431,  432,   -1,   -1,
  379,   -1,  437,  438,  439,  440,  349,   -1,   -1,  388,
   -1,  390,  257,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  780,  781,  782,   -1,  784,  785,  786,  787,
  373,  374,  375,  376,  377,  378,  259,   -1,  349,   -1,
   -1,   -1,  385,  386,  387,   -1,  804,   -1,  293,  294,
   -1,  809,  259,   -1,   -1,  372,   -1,   -1,   -1,  817,
  305,  306,  373,  374,  375,  376,  377,  378,  313,   -1,
  413,  414,  415,  416,  385,  386,  387,   -1,   -1,  396,
  397,   -1,  399,   -1,   -1,  402,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  419,  420,   -1,   -1,   -1,  424,  425,   -1,
  427,   -1,  429,   -1,  431,  432,   -1,   -1,   -1,   -1,
  437,  438,  439,  440,   -1,   -1,  349,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  379,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  349,  388,   -1,  390,   -1,  370,   -1,   -1,
  373,  374,  375,  376,  377,  378,   -1,   -1,   -1,   -1,
  383,  384,  385,  386,  387,   -1,  373,  374,  375,  376,
  377,  378,   -1,   -1,   -1,   -1,   -1,   -1,  385,  386,
  387,  314,  315,  316,  317,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,  331,  332,
  333,  334,  335,  336,  337,  338,  339,
};
#define YYFINAL 10
#ifndef YYDEBUG
#define YYDEBUG 1
#endif
#define YYMAXTOKEN 450
#if YYDEBUG
static const char *yyname[] = {

"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,"'@'",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"IF","ELSE",
"THEN","OR","FAILED","SET","LOGFILE","FACILITY","DAEMON","SYSLOG","MAILSERVER",
"HTTPD","ALLOW","ADDRESS","INIT","READONLY","CLEARTEXT","MD5HASH","SHA1HASH",
"CRYPT","DELAY","PEMFILE","ENABLE","DISABLE","HTTPDSSL","CLIENTPEMFILE",
"ALLOWSELFCERTIFICATION","IDFILE","STATEFILE","SEND","EXPECT","EXPECTBUFFER",
"CYCLE","COUNT","REMINDER","PIDFILE","START","STOP","PATHTOK","HOST","HOSTNAME",
"PORT","TYPE","UDP","TCP","TCPSSL","PROTOCOL","CONNECTION","ALERT","NOALERT",
"MAILFORMAT","UNIXSOCKET","SIGNATURE","TIMEOUT","RESTART","CHECKSUM","EVERY",
"DEFAULT","HTTP","APACHESTATUS","FTP","SMTP","POP","IMAP","CLAMAV","NNTP",
"NTP3","MYSQL","DNS","SSH","DWP","LDAP2","LDAP3","RDATE","RSYNC","TNS","PGSQL",
"POSTFIXPOLICY","SIP","LMTP","GPS","RADIUS","MEMCACHE","STRING","PATH",
"MAILADDR","MAILFROM","MAILREPLYTO","MAILSUBJECT","MAILBODY","SERVICENAME",
"STRINGNAME","NUMBER","PERCENT","LOGLIMIT","CLOSELIMIT","DNSLIMIT",
"KEEPALIVELIMIT","REPLYLIMIT","REQUESTLIMIT","STARTLIMIT","WAITLIMIT",
"GRACEFULLIMIT","CLEANUPLIMIT","REAL","CHECKPROC","CHECKFILESYS","CHECKFILE",
"CHECKDIR","CHECKHOST","CHECKSYSTEM","CHECKFIFO","CHECKSTATUS","CHILDREN",
"SYSTEM","RESOURCE","MEMORY","TOTALMEMORY","LOADAVG1","LOADAVG5","LOADAVG15",
"SWAP","MODE","ACTIVE","PASSIVE","MANUAL","CPU","TOTALCPU","CPUUSER",
"CPUSYSTEM","CPUWAIT","GROUP","REQUEST","DEPENDS","BASEDIR","SLOT","EVENTQUEUE",
"SECRET","HOSTHEADER","UID","GID","MMONIT","INSTANCE","USERNAME","PASSWORD",
"TIMESTAMP","CHANGED","SECOND","MINUTE","HOUR","DAY","SSLAUTO","SSLV2","SSLV3",
"TLSV1","CERTMD5","BYTE","KILOBYTE","MEGABYTE","GIGABYTE","INODE","SPACE",
"PERMISSION","SIZE","MATCH","NOT","IGNORE","ACTION","EXEC","UNMONITOR","ICMP",
"ICMPECHO","NONEXIST","EXIST","INVALID","DATA","RECOVERED","PASSED","SUCCEEDED",
"URL","CONTENT","PID","PPID","FSFLAG","REGISTER","CREDENTIALS","URLOBJECT",
"TARGET","MAXFORWARD","FIPS","GREATER","LESS","EQUAL","NOTEQUAL",
};
static const char *yyrule[] = {
"$accept : cfgfile",
"cfgfile :",
"cfgfile : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : setalert",
"statement : setdaemon",
"statement : setlog",
"statement : seteventqueue",
"statement : setmmonits",
"statement : setmailservers",
"statement : setmailformat",
"statement : sethttpd",
"statement : setpid",
"statement : setidfile",
"statement : setstatefile",
"statement : setexpectbuffer",
"statement : setinit",
"statement : setfips",
"statement : checkproc optproclist",
"statement : checkfile optfilelist",
"statement : checkfilesys optfilesyslist",
"statement : checkdir optdirlist",
"statement : checkhost opthostlist",
"statement : checksystem optsystemlist",
"statement : checkfifo optfifolist",
"statement : checkstatus optstatuslist",
"optproclist :",
"optproclist : optproclist optproc",
"optproc : start",
"optproc : stop",
"optproc : exist",
"optproc : pid",
"optproc : ppid",
"optproc : connection",
"optproc : connectionunix",
"optproc : actionrate",
"optproc : alert",
"optproc : every",
"optproc : mode",
"optproc : group",
"optproc : depend",
"optproc : resourceprocess",
"optfilelist :",
"optfilelist : optfilelist optfile",
"optfile : start",
"optfile : stop",
"optfile : exist",
"optfile : timestamp",
"optfile : actionrate",
"optfile : every",
"optfile : alert",
"optfile : permission",
"optfile : uid",
"optfile : gid",
"optfile : checksum",
"optfile : size",
"optfile : match",
"optfile : mode",
"optfile : group",
"optfile : depend",
"optfilesyslist :",
"optfilesyslist : optfilesyslist optfilesys",
"optfilesys : start",
"optfilesys : stop",
"optfilesys : exist",
"optfilesys : actionrate",
"optfilesys : every",
"optfilesys : alert",
"optfilesys : permission",
"optfilesys : uid",
"optfilesys : gid",
"optfilesys : mode",
"optfilesys : group",
"optfilesys : depend",
"optfilesys : inode",
"optfilesys : space",
"optfilesys : fsflag",
"optdirlist :",
"optdirlist : optdirlist optdir",
"optdir : start",
"optdir : stop",
"optdir : exist",
"optdir : timestamp",
"optdir : actionrate",
"optdir : every",
"optdir : alert",
"optdir : permission",
"optdir : uid",
"optdir : gid",
"optdir : mode",
"optdir : group",
"optdir : depend",
"opthostlist : opthost",
"opthostlist : opthostlist opthost",
"opthost : start",
"opthost : stop",
"opthost : exist",
"opthost : connection",
"opthost : icmp",
"opthost : actionrate",
"opthost : alert",
"opthost : every",
"opthost : mode",
"opthost : group",
"opthost : depend",
"optsystemlist :",
"optsystemlist : optsystemlist optsystem",
"optsystem : start",
"optsystem : stop",
"optsystem : actionrate",
"optsystem : alert",
"optsystem : every",
"optsystem : group",
"optsystem : depend",
"optsystem : resourcesystem",
"optfifolist :",
"optfifolist : optfifolist optfifo",
"optfifo : start",
"optfifo : stop",
"optfifo : exist",
"optfifo : timestamp",
"optfifo : actionrate",
"optfifo : every",
"optfifo : alert",
"optfifo : permission",
"optfifo : uid",
"optfifo : gid",
"optfifo : mode",
"optfifo : group",
"optfifo : depend",
"optstatuslist :",
"optstatuslist : optstatuslist optstatus",
"optstatus : actionrate",
"optstatus : exist",
"optstatus : alert",
"optstatus : every",
"optstatus : group",
"optstatus : depend",
"setalert : SET alertmail '{' eventoptionlist '}' formatlist reminder",
"setalert : SET alertmail formatlist reminder",
"setalert : SET alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"setdaemon : SET DAEMON NUMBER startdelay",
"startdelay :",
"startdelay : START DELAY NUMBER",
"setexpectbuffer : SET EXPECTBUFFER NUMBER unit",
"setinit : SET INIT",
"setfips : SET FIPS",
"setlog : SET LOGFILE PATH",
"setlog : SET LOGFILE SYSLOG",
"setlog : SET LOGFILE SYSLOG FACILITY STRING",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH SLOT NUMBER",
"seteventqueue : SET EVENTQUEUE SLOT NUMBER",
"setidfile : SET IDFILE PATH",
"setstatefile : SET STATEFILE PATH",
"setpid : SET PIDFILE PATH",
"setmmonits : SET MMONIT mmonitlist",
"mmonitlist : mmonit credentials",
"mmonitlist : mmonitlist mmonit credentials",
"mmonit : URLOBJECT nettimeout sslversion certmd5",
"credentials :",
"credentials : REGISTER CREDENTIALS",
"setmailservers : SET MAILSERVER mailserverlist nettimeout hostname",
"setmailformat : SET MAILFORMAT '{' formatoptionlist '}'",
"sethttpd : SET HTTPD PORT NUMBER httpdlist",
"mailserverlist : mailserver",
"mailserverlist : mailserverlist mailserver",
"mailserver : STRING username password sslversion certmd5",
"mailserver : STRING PORT NUMBER username password sslversion certmd5",
"httpdlist :",
"httpdlist : httpdlist httpdoption",
"httpdoption : ssl",
"httpdoption : signature",
"httpdoption : bindaddress",
"httpdoption : allow",
"ssl : ssldisable",
"ssl : sslenable pemfile clientpemfile allowselfcert",
"sslenable : HTTPDSSL",
"sslenable : HTTPDSSL ENABLE",
"sslenable : ENABLE HTTPDSSL",
"ssldisable : HTTPDSSL DISABLE",
"ssldisable : DISABLE HTTPDSSL",
"ssldisable : ssldisable PEMFILE PATH",
"ssldisable : ssldisable CLIENTPEMFILE PATH",
"ssldisable : ssldisable ALLOWSELFCERTIFICATION",
"signature : sigenable",
"signature : sigdisable",
"sigenable : SIGNATURE ENABLE",
"sigenable : ENABLE SIGNATURE",
"sigdisable : SIGNATURE DISABLE",
"sigdisable : DISABLE SIGNATURE",
"bindaddress : ADDRESS STRING",
"pemfile : PEMFILE PATH",
"clientpemfile :",
"clientpemfile : CLIENTPEMFILE PATH",
"allowselfcert :",
"allowselfcert : ALLOWSELFCERTIFICATION",
"allow : ALLOW STRING ':' STRING readonly",
"allow : ALLOW '@' STRING readonly",
"allow : ALLOW PATH",
"allow : ALLOW CLEARTEXT PATH",
"allow : ALLOW MD5HASH PATH",
"allow : ALLOW CRYPT PATH",
"$$1 :",
"allow : ALLOW PATH $$1 allowuserlist",
"$$2 :",
"allow : ALLOW CLEARTEXT PATH $$2 allowuserlist",
"$$3 :",
"allow : ALLOW MD5HASH PATH $$3 allowuserlist",
"$$4 :",
"allow : ALLOW CRYPT PATH $$4 allowuserlist",
"allow : ALLOW STRING",
"allowuserlist : allowuser",
"allowuserlist : allowuserlist allowuser",
"allowuser : STRING",
"readonly :",
"readonly : READONLY",
"checkproc : CHECKPROC SERVICENAME PIDFILE PATH",
"checkproc : CHECKPROC SERVICENAME PATHTOK PATH",
"checkproc : CHECKPROC SERVICENAME MATCH STRING",
"checkproc : CHECKPROC SERVICENAME MATCH PATH",
"checkfile : CHECKFILE SERVICENAME PATHTOK PATH",
"checkfilesys : CHECKFILESYS SERVICENAME PATHTOK PATH",
"checkdir : CHECKDIR SERVICENAME PATHTOK PATH",
"checkhost : CHECKHOST SERVICENAME ADDRESS STRING",
"checksystem : CHECKSYSTEM SERVICENAME",
"checkfifo : CHECKFIFO SERVICENAME PATHTOK PATH",
"checkstatus : CHECKSTATUS SERVICENAME PATHTOK PATH",
"start : START argumentlist exectimeout",
"start : START argumentlist useroptionlist exectimeout",
"stop : STOP argumentlist exectimeout",
"stop : STOP argumentlist useroptionlist exectimeout",
"argumentlist : argument",
"argumentlist : argumentlist argument",
"useroptionlist : useroption",
"useroptionlist : useroptionlist useroption",
"argument : STRING",
"argument : PATH",
"useroption : UID STRING",
"useroption : GID STRING",
"useroption : UID NUMBER",
"useroption : GID NUMBER",
"username :",
"username : USERNAME MAILADDR",
"username : USERNAME STRING",
"password :",
"password : PASSWORD STRING",
"hostname :",
"hostname : HOSTNAME STRING",
"connection : IF FAILED host port type protocol nettimeout rate1 THEN action1 recovery",
"connection : IF FAILED URL URLOBJECT urloption nettimeout rate1 THEN action1 recovery",
"connectionunix : IF FAILED unixsocket type protocol nettimeout rate1 THEN action1 recovery",
"icmp : IF FAILED ICMP icmptype icmpcount nettimeout rate1 THEN action1 recovery",
"host :",
"host : HOST STRING",
"port : PORT NUMBER",
"unixsocket : UNIXSOCKET PATH",
"type :",
"type : TYPE TCP",
"type : TYPE TCPSSL sslversion certmd5",
"type : TYPE UDP",
"certmd5 :",
"certmd5 : CERTMD5 STRING",
"sslversion :",
"sslversion : SSLV2",
"sslversion : SSLV3",
"sslversion : TLSV1",
"sslversion : SSLAUTO",
"protocol :",
"protocol : PROTOCOL APACHESTATUS apache_stat_list",
"protocol : PROTOCOL DEFAULT",
"protocol : PROTOCOL DNS",
"protocol : PROTOCOL DWP",
"protocol : PROTOCOL FTP",
"protocol : PROTOCOL HTTP request",
"protocol : PROTOCOL IMAP",
"protocol : PROTOCOL CLAMAV",
"protocol : PROTOCOL LDAP2",
"protocol : PROTOCOL LDAP3",
"protocol : PROTOCOL MYSQL",
"protocol : PROTOCOL SIP target maxforward",
"protocol : PROTOCOL NNTP",
"protocol : PROTOCOL NTP3",
"protocol : PROTOCOL POSTFIXPOLICY",
"protocol : PROTOCOL POP",
"protocol : PROTOCOL SMTP",
"protocol : PROTOCOL SSH",
"protocol : PROTOCOL RDATE",
"protocol : PROTOCOL RSYNC",
"protocol : PROTOCOL TNS",
"protocol : PROTOCOL PGSQL",
"protocol : PROTOCOL LMTP",
"protocol : PROTOCOL GPS",
"protocol : PROTOCOL RADIUS secret",
"protocol : PROTOCOL MEMCACHE",
"protocol : sendexpectlist",
"sendexpectlist : sendexpect",
"sendexpectlist : sendexpectlist sendexpect",
"sendexpect : SEND STRING",
"sendexpect : EXPECT STRING",
"target :",
"target : TARGET MAILADDR",
"target : TARGET STRING",
"maxforward :",
"maxforward : MAXFORWARD NUMBER",
"request :",
"request : REQUEST PATH hostheader",
"request : REQUEST PATH CHECKSUM STRING hostheader",
"hostheader :",
"hostheader : HOSTHEADER STRING",
"secret : SECRET STRING",
"apache_stat_list : apache_stat",
"apache_stat_list : apache_stat_list OR apache_stat",
"apache_stat : LOGLIMIT operator NUMBER PERCENT",
"apache_stat : CLOSELIMIT operator NUMBER PERCENT",
"apache_stat : DNSLIMIT operator NUMBER PERCENT",
"apache_stat : KEEPALIVELIMIT operator NUMBER PERCENT",
"apache_stat : REPLYLIMIT operator NUMBER PERCENT",
"apache_stat : REQUESTLIMIT operator NUMBER PERCENT",
"apache_stat : STARTLIMIT operator NUMBER PERCENT",
"apache_stat : WAITLIMIT operator NUMBER PERCENT",
"apache_stat : GRACEFULLIMIT operator NUMBER PERCENT",
"apache_stat : CLEANUPLIMIT operator NUMBER PERCENT",
"exist : IF NOT EXIST rate1 THEN action1 recovery",
"pid : IF CHANGED PID rate1 THEN action1",
"ppid : IF CHANGED PPID rate1 THEN action1",
"icmpcount :",
"icmpcount : COUNT NUMBER",
"exectimeout :",
"exectimeout : TIMEOUT NUMBER SECOND",
"nettimeout :",
"nettimeout : TIMEOUT NUMBER SECOND",
"actionrate : IF NUMBER RESTART NUMBER CYCLE THEN action1",
"actionrate : IF NUMBER RESTART NUMBER CYCLE THEN TIMEOUT",
"urloption :",
"urloption : CONTENT urloperator STRING",
"urloperator : EQUAL",
"urloperator : NOTEQUAL",
"alert : alertmail '{' eventoptionlist '}' formatlist reminder",
"alert : alertmail formatlist reminder",
"alert : alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"alert : noalertmail",
"alertmail : ALERT MAILADDR",
"noalertmail : NOALERT MAILADDR",
"eventoptionlist : eventoption",
"eventoptionlist : eventoptionlist eventoption",
"eventoption : ACTION",
"eventoption : CHECKSUM",
"eventoption : CONNECTION",
"eventoption : CONTENT",
"eventoption : DATA",
"eventoption : EXEC",
"eventoption : FSFLAG",
"eventoption : GID",
"eventoption : ICMP",
"eventoption : INSTANCE",
"eventoption : INVALID",
"eventoption : NONEXIST",
"eventoption : PERMISSION",
"eventoption : PID",
"eventoption : PPID",
"eventoption : RESOURCE",
"eventoption : SIZE",
"eventoption : TIMEOUT",
"eventoption : TIMESTAMP",
"eventoption : UID",
"formatlist :",
"formatlist : MAILFORMAT '{' formatoptionlist '}'",
"formatoptionlist : formatoption",
"formatoptionlist : formatoptionlist formatoption",
"formatoption : MAILFROM",
"formatoption : MAILREPLYTO",
"formatoption : MAILSUBJECT",
"formatoption : MAILBODY",
"every : EVERY NUMBER CYCLE",
"mode : MODE ACTIVE",
"mode : MODE PASSIVE",
"mode : MODE MANUAL",
"group : GROUP STRINGNAME",
"depend : DEPENDS dependlist",
"dependlist : dependant",
"dependlist : dependlist dependant",
"dependant : SERVICENAME",
"resourceprocess : IF resourceprocesslist rate1 THEN action1 recovery",
"resourceprocesslist : resourceprocessopt",
"resourceprocesslist : resourceprocesslist resourceprocessopt",
"resourceprocessopt : resourcecpuproc",
"resourceprocessopt : resourcemem",
"resourceprocessopt : resourcechild",
"resourceprocessopt : resourceload",
"resourcesystem : IF resourcesystemlist rate1 THEN action1 recovery",
"resourcesystemlist : resourcesystemopt",
"resourcesystemlist : resourcesystemlist resourcesystemopt",
"resourcesystemopt : resourceload",
"resourcesystemopt : resourcemem",
"resourcesystemopt : resourceswap",
"resourcesystemopt : resourcecpu",
"resourcecpuproc : CPU operator NUMBER PERCENT",
"resourcecpuproc : TOTALCPU operator NUMBER PERCENT",
"resourcecpu : resourcecpuid operator NUMBER PERCENT",
"resourcecpuid : CPUUSER",
"resourcecpuid : CPUSYSTEM",
"resourcecpuid : CPUWAIT",
"resourcemem : MEMORY operator value unit",
"resourcemem : MEMORY operator NUMBER PERCENT",
"resourcemem : TOTALMEMORY operator value unit",
"resourcemem : TOTALMEMORY operator NUMBER PERCENT",
"resourceswap : SWAP operator value unit",
"resourceswap : SWAP operator NUMBER PERCENT",
"resourcechild : CHILDREN operator NUMBER",
"resourceload : resourceloadavg operator value",
"resourceloadavg : LOADAVG1",
"resourceloadavg : LOADAVG5",
"resourceloadavg : LOADAVG15",
"value : REAL",
"value : NUMBER",
"timestamp : IF TIMESTAMP operator NUMBER time rate1 THEN action1 recovery",
"timestamp : IF CHANGED TIMESTAMP rate1 THEN action1",
"operator :",
"operator : GREATER",
"operator : LESS",
"operator : EQUAL",
"operator : NOTEQUAL",
"operator : CHANGED",
"time :",
"time : SECOND",
"time : MINUTE",
"time : HOUR",
"time : DAY",
"action : ALERT",
"action : EXEC argumentlist",
"action : EXEC argumentlist useroptionlist",
"action : RESTART",
"action : START",
"action : STOP",
"action : UNMONITOR",
"action1 : action",
"action2 : action",
"rate1 :",
"rate1 : NUMBER CYCLE",
"rate1 : NUMBER NUMBER CYCLE",
"rate2 :",
"rate2 : NUMBER CYCLE",
"rate2 : NUMBER NUMBER CYCLE",
"recovery :",
"recovery : ELSE IF RECOVERED rate2 THEN action2",
"recovery : ELSE IF PASSED rate2 THEN action2",
"recovery : ELSE IF SUCCEEDED rate2 THEN action2",
"checksum : IF FAILED hashtype CHECKSUM rate1 THEN action1 recovery",
"checksum : IF FAILED hashtype CHECKSUM EXPECT STRING rate1 THEN action1 recovery",
"checksum : IF CHANGED hashtype CHECKSUM rate1 THEN action1",
"hashtype :",
"hashtype : MD5HASH",
"hashtype : SHA1HASH",
"inode : IF INODE operator NUMBER rate1 THEN action1 recovery",
"inode : IF INODE operator NUMBER PERCENT rate1 THEN action1 recovery",
"space : IF SPACE operator value unit rate1 THEN action1 recovery",
"space : IF SPACE operator NUMBER PERCENT rate1 THEN action1 recovery",
"fsflag : IF CHANGED FSFLAG rate1 THEN action1",
"unit :",
"unit : BYTE",
"unit : KILOBYTE",
"unit : MEGABYTE",
"unit : GIGABYTE",
"permission : IF FAILED PERMISSION NUMBER rate1 THEN action1 recovery",
"match : IF matchflagnot MATCH PATH rate1 THEN action1",
"match : IF matchflagnot MATCH STRING rate1 THEN action1",
"match : IGNORE matchflagnot MATCH PATH",
"match : IGNORE matchflagnot MATCH STRING",
"matchflagnot :",
"matchflagnot : NOT",
"size : IF SIZE operator NUMBER unit rate1 THEN action1 recovery",
"size : IF CHANGED SIZE rate1 THEN action1",
"uid : IF FAILED UID STRING rate1 THEN action1 recovery",
"uid : IF FAILED UID NUMBER rate1 THEN action1 recovery",
"gid : IF FAILED GID STRING rate1 THEN action1 recovery",
"gid : IF FAILED GID NUMBER rate1 THEN action1 recovery",
"icmptype : TYPE ICMPECHO",
"reminder :",
"reminder : REMINDER NUMBER",
"reminder : REMINDER NUMBER CYCLE",

};
#endif
/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH  500
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;

typedef struct {
    unsigned stacksize;
    short    *s_base;
    short    *s_mark;
    short    *s_last;
    YYSTYPE  *l_base;
    YYSTYPE  *l_mark;
} YYSTACKDATA;
int      yyerrflag;
int      yychar;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static YYSTACKDATA yystack;
#line 1703 "p.y"


/* -------------------------------------------------------- Parser interface */


/**
 * Syntactic error routine
 *
 * This routine is automatically called by the lexer!
 */
void yyerror(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", currentfile, lineno, msg, yytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Syntactical warning routine
 */
void yywarning(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", currentfile, lineno, msg, yytext);
 
  FREE(msg);
  
}

/**
 * Argument error routine
 */
void yyerror2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Argument warning routine
 */
void yywarning2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
 
  FREE(msg);
  
}

/*
 * The Parser hook - start parsing the control file
 * Returns TRUE if parsing succeeded, otherwise FALSE
 */
int parse(char *controlfile) {

  ASSERT(controlfile);

  servicelist = tail = current = NULL;

  /*
   * Secure check the monitrc file. The run control file must have the
   * same uid as the REAL uid of this process, it must have permissions
   * no greater than 700 and it must not be a symbolic link.
   */
  if (! File_checkStat(controlfile, "control file", S_IRUSR|S_IWUSR|S_IXUSR))
    return FALSE;

  if ((yyin = fopen(controlfile,"r")) == (FILE *)NULL) {
    LogError("%s: Error: cannot open the control file '%s' -- %s\n", prog, controlfile, STRERROR);
    return FALSE;
  }

  currentfile = xstrdup(controlfile);

  /* 
   * Creation of the global service list is synchronized  
   */
  LOCK(Run.mutex)
    preparse();
    yyparse();
    fclose(yyin);
    /* Add the default general system service if not specified explicitly */
    if (!hassystem) {
      char *name = Util_getString("system_%s", Run.localhostname);
      if (Util_existService(name) || (current && IS(name, current->name))) {
        LogError("'check system' not defined in control file, failed to add automatic configuration (service name %s is used already) -- please add 'check system <name>' manually\n", name, name);
        FREE(name);
        cfg_errflag++;
      } else {
        createservice(TYPE_SYSTEM, name, xstrdup(""), check_system);
      }
    }
    /* If defined - add the last service to the service list */
    if (current) {
      addservice(current);
      FREE(current);
    }
    postparse();
  END_LOCK;

  FREE(currentfile);

  if (argyytext != NULL)
    FREE(argyytext);

  return(cfg_errflag == 0);
}


/* ----------------------------------------------------------------- Private */


/**
 * Initialize objects used by the parser.
 */
static void preparse() {
  int i;
  char localhost[STRLEN];

  /*
   * Get the localhost name
   */
  if (gethostname(localhost, sizeof(localhost)) < 0)
    snprintf(localhost, STRLEN, "%s", LOCALHOST);

  /* Set instance incarnation ID */
  time(&Run.incarnation);
  /* Reset lexer */
  buffer_stack_ptr        = 0;
  lineno                  = 1;
  arglineno               = 1;
  argcurrentfile          = NULL;
  argyytext               = NULL;
  /* Reset parser */
  Run.stopped             = FALSE;
  Run.dolog               = FALSE;
  Run.dohttpd             = FALSE;
  Run.doaction            = FALSE;
  Run.httpdsig            = TRUE;
  Run.dommonitcredentials = TRUE;
  Run.mmonitcredentials   = NULL;
  Run.credentials         = NULL;
  Run.httpdssl            = FALSE;
  Run.httpsslpem          = NULL;
  Run.httpsslclientpem    = NULL;
  Run.clientssl           = FALSE;
  Run.mailserver_timeout  = NET_TIMEOUT;
  Run.bind_addr           = NULL;
  Run.eventlist           = NULL;
  Run.eventlist_dir       = NULL;
  Run.eventlist_slots     = -1;
  Run.system              = NULL;
  Run.expectbuffer        = STRLEN;
  Run.mmonits             = NULL;
  Run.maillist            = NULL;
  Run.mailservers         = NULL;
  Run.MailFormat.from     = NULL;
  Run.MailFormat.replyto  = NULL;
  Run.MailFormat.subject  = NULL;
  Run.MailFormat.message  = NULL;
  Run.localhostname       = xstrdup(localhost);
  depend_list             = NULL;
  Run.handler_init        = TRUE;
#ifdef OPENSSL_FIPS  
  Run.fipsEnabled         = FALSE;
#endif
  for (i = 0; i <= HANDLER_MAX; i++)
    Run.handler_queue[i] = 0;
  /* 
   * Initialize objects
   */
  reset_uidset();
  reset_gidset();
  reset_sizeset();
  reset_mailset();
  reset_mailserverset();
  reset_portset();
  reset_permset();
  reset_icmpset();
  reset_rateset();
  reset_filesystemset();
  reset_resourceset();
  reset_checksumset();
  reset_timestampset();
  reset_actionrateset();
}


/*
 * Check that values are reasonable after parsing
 */
static void postparse() {
  Service_T s;

  if (cfg_errflag || ! servicelist)
    return;

  /* Check the sanity of any dependency graph */
  check_depend();

  /* Check that we do not start monit in daemon mode without having a
   * poll time */
  if (!Run.polltime && (Run.isdaemon || Run.init)) {
    LogError("%s: Error: Poll time not defined. Please define poll time in the\n control file or use the -d option when starting monit\n", prog);
    cfg_errflag++;
  }

  if (Run.logfile)
    Run.dolog = TRUE;

  for (s = servicelist; s; s = s->next) {
    /* Set the general system service shortcut */
    if (s->type == TYPE_SYSTEM)
      Run.system = s;
    if (s->type != TYPE_HOST)
	continue;
    /* Verify that a remote service has a port or an icmp list */
    if (!s->portlist && !s->icmplist) {
      LogError("%s: Error: 'check host' statement is incomplete; Please specify a port number to test\n or an icmp test at the remote host: '%s'\n", prog, s->name);
      cfg_errflag++;
    }
  }

  if (Run.mmonits) {
    if (Run.dohttpd) {
      if (Run.dommonitcredentials) {
        Auth_T c;
        for (c = Run.credentials; c; c = c->next) {
          if (c->digesttype == DIGEST_CLEARTEXT && ! c->is_readonly) {
              Run.mmonitcredentials = c;
              break;
          }
        }
        if (! Run.mmonitcredentials)
          LogWarning("%s: Warning: M/Monit registration with credentials enabled, but no suitable credentials found in monit configuration file -- please add 'allow user:password' option to 'set httpd' statement\n", prog);
      }
    } else
        LogWarning("%s: Warning: M/Monit enabled but no httpd allowed -- please add 'set httpd' statement\n", prog);
  }
}


/*
 * Create a new service object and add any current objects to the
 * service list.
 */
static void createservice(int type, char *name, char *value, int (*check)(Service_T s)) {

  ASSERT(name);
  ASSERT(value);

  check_name(name);

  if (current)
    addservice(current);
  else
    NEW(current);

  /* Reset the current object */
  memset(current, 0, sizeof(*current));

  NEW(current->inf);
  Util_resetInfo(current);

  /* Set default values */
  current->monitor = MONITOR_INIT;
  current->mode    = MODE_ACTIVE;
  current->name    = name;
  current->type    = type;
  current->check   = check;
  current->path    = value;

  /* Initialize general event handlers */
  addeventaction(&(current)->action_DATA,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_EXEC,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_INVALID,  ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_NONEXIST, ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_PID,      ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_PPID,     ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_FSFLAG,   ACTION_ALERT,     ACTION_IGNORE);

  /* Initialize internal event handlers */
  addeventaction(&(current)->action_MONIT_START,  ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_STOP,   ACTION_STOP,  ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_RELOAD, ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_ACTION,       ACTION_ALERT, ACTION_IGNORE);
  
  gettimeofday(&current->collected, NULL);

}


/*
 * Add a service object to the servicelist
 */
static void addservice(Service_T s) {
  Service_T n;

  ASSERT(s);
 
  NEW(n);
  memcpy(n, s, sizeof(*s));
  /* Add the service to the end of the service list */
  if (tail != NULL) {
    tail->next = n;
    tail->next_conf = n;
  } else {
    servicelist = n;
    servicelist_conf = n;
  }
  tail = n;
}


/* 
 * Add entry to service group list
 */
static void addservicegroup(char *name) {
  ServiceGroup_T g;
  ServiceGroupMember_T m;

  ASSERT(name);
 
  /* Check if service group with the same name is defined already */
  for (g = servicegrouplist; g; g = g->next)
    if (! strcasecmp(g->name, name))
      break;

  if (! g) {
    NEW(g);
    g->name = xstrdup(name);
    g->next = servicegrouplist;
    servicegrouplist = g;
  }

  NEW(m);
  m->name = xstrdup(current->name);
  m->next = g->members;
  g->members = m;
}


/* 
 * Add a dependant entry to the current service dependant list
 *
 */
static void adddependant(char *dependant) {
  Dependant_T d; 

  ASSERT(dependant);
  
  NEW(d);
  
  if (current->dependantlist != NULL)
    d->next = current->dependantlist;

  d->dependant = dependant;
  current->dependantlist = d;

}


/*
 * Add the given mailaddress with the apropriate alert notification
 * values and mail attributes to the given mailinglist.
 */
static void addmail(char *mailto, Mail_T f, Mail_T *l, unsigned int events, unsigned int reminder) {
  Mail_T m;

  ASSERT(mailto);

  NEW(m);
  m->events   = events;
  m->to       = mailto;
  m->from     = f->from;
  m->subject  = f->subject;
  m->message  = f->message;
  m->reminder = reminder;
  
  m->next = *l;
  *l = m;

  reset_mailset();
}


/*
 * Add the given portset to the current service's portlist
 */
static void addport(Port_T port) {
  Port_T p;
  char address[STRLEN];
  
  ASSERT(port);

  NEW(p);
  p->port               = port->port;
  p->type               = port->type;
  p->socket             = port->socket;
  p->family             = port->family;
  p->action             = port->action;
  p->timeout            = port->timeout;
  p->request            = port->request;
  p->generic            = port->generic;
  p->protocol           = port->protocol;
  p->pathname           = port->pathname;
  p->hostname           = port->hostname;
  p->url_request        = port->url_request;
  p->request_checksum   = port->request_checksum;
  p->request_hostheader = port->request_hostheader;
  memcpy(&p->ApacheStatus, &port->ApacheStatus, sizeof(struct apache_status));

  if (p->request_checksum) {
    cleanup_hash_string(p->request_checksum);
    if (strlen(p->request_checksum) == 32)
      p->request_hashtype = HASH_MD5;
    else if (strlen(p->request_checksum) == 40)
      p->request_hashtype = HASH_SHA1;
    else
      yyerror2("invalid checksum [%s]", p->request_checksum);
  } else
    p->request_hashtype = 0;

  if (port->family == AF_INET)
    snprintf(address, STRLEN, "INET[%s:%d]", port->hostname, port->port);
  else if (port->family == AF_UNIX)
    snprintf(address, STRLEN, "UNIX[%s]", port->pathname);
  else
    address[0] = 0;
  p->address = xstrdup(address);
  
  if (port->SSL.use_ssl == TRUE) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      if (port->SSL.certmd5 != NULL) {
	p->SSL.certmd5 = port->SSL.certmd5;
	cleanup_hash_string(p->SSL.certmd5);
      }
      p->SSL.use_ssl = TRUE;
      p->SSL.version = port->SSL.version;
    }
  }
  p->maxforward = port->maxforward;
  p->next = current->portlist;
  current->portlist = p;
  
  reset_portset();

}


/*
 * Add a new resource object to the current service resource list
 */
static void addresource(Resource_T rr) {
  Resource_T r;

  ASSERT(rr);

  NEW(r);
  if (! Run.doprocess)
    yyerror("Cannot activate service check. The process status engine was disabled. On certain systems you must run monit as root to utilize this feature)\n");
  r->resource_id = rr->resource_id;
  r->limit       = rr->limit;
  r->action      = rr->action;
  r->operator    = rr->operator;
  r->next        = current->resourcelist;

  current->resourcelist = r;
  reset_resourceset();
}


/*
 * Add a new file object to the current service timestamp list
 */
static void addtimestamp(Timestamp_T ts, int notime) {
  Timestamp_T t;

  ASSERT(ts);

  NEW(t);
  t->operator     = ts->operator;
  t->time         = ts->time;
  t->action       = ts->action;
  t->test_changes = ts->test_changes;
  
  if (t->test_changes || notime) {
    if (! File_exist(current->path)) {
      DEBUG("%s: Debug: the path '%s' used in the TIMESTAMP statement refer to a non-existing object\n", prog, current->path);
    } else if (!(t->timestamp = File_getTimestamp(current->path, S_IFDIR|S_IFREG))) {
      yyerror2("cannot get the timestamp for '%s'", current->path);
    }
  }
  
  t->next = current->timestamplist;
  current->timestamplist = t;

  reset_timestampset();
}


/*
 * Add a new object to the current service actionrate list
 */
static void addactionrate(ActionRate_T ar) {
  ActionRate_T a;

  ASSERT(ar);

  if (ar->count > ar->cycle)
    yyerror2("the number of restarts must be less than poll cycles");
  if (ar->count <= 0 || ar->cycle <= 0)
    yyerror2("zero or negative values not allowed in a action rate statement");

  NEW(a);
  a->count  = ar->count;
  a->cycle  = ar->cycle;
  a->action = ar->action;

  a->next = current->actionratelist;
  current->actionratelist = a;

  reset_actionrateset();
}



/*
 * Add a new Size object to the current service size list
 */
static void addsize(Size_T ss) {
  Size_T s;
  struct stat buf;

  ASSERT(ss);

  NEW(s);
  s->operator     = ss->operator;
  s->size         = ss->size;
  s->action       = ss->action;
  s->test_changes = ss->test_changes;
  /* Get the initial size for future comparision, if the file exists */
  if (s->test_changes) {
    s->test_changes_ok = !stat(current->path, &buf);
    if (s->test_changes_ok)
      s->size = (unsigned long long)buf.st_size;
  }
 
  s->next = current->sizelist;
  current->sizelist = s;

  reset_sizeset();
}


/*
 * Set Checksum object in the current service
 */
static void addchecksum(Checksum_T cs) {

  int len;
  Checksum_T c;

  ASSERT(cs);

  cs->test_changes_ok = TRUE;

  if (! *cs->hash) {
    if (cs->type == HASH_UNKNOWN)
      cs->type = DEFAULT_HASH;
    if ( !(Util_getChecksum(current->path, cs->type, cs->hash, sizeof(cs->hash)))) {
      if (cs->test_changes == TRUE) {
        /* If the file doesn't exist and we're checking for checksum changes, set dummy value */
        cs->test_changes_ok = FALSE;
        snprintf(cs->hash, sizeof(cs->hash), "00000000000000000000000000000000");
      } else {
        yyerror2("cannot compute a checksum for file %s", current->path);
        reset_checksumset();
        return;
      }
    }
  }

  len = cleanup_hash_string(cs->hash);

  if (cs->type == HASH_UNKNOWN) {
    if (len == 32) {
      cs->type = HASH_MD5;
    } else if (len == 40) {
      cs->type = HASH_SHA1;
    } else {
      yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
      reset_checksumset();
      return;
    }
  } else if (( cs->type == HASH_MD5 && len!=32 ) || ( cs->type == HASH_SHA1 && len != 40 )) {
    yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
    reset_checksumset();
    return;
  }

  NEW(c);

  c->type            = cs->type;
  c->test_changes    = cs->test_changes;
  c->test_changes_ok = cs->test_changes_ok;
  c->action          = cs->action;
  snprintf(c->hash, sizeof(c->hash), "%s", cs->hash);
 
  current->checksum = c;

  reset_checksumset();

}


/*
 * Set Perm object in the current service
 */
static void addperm(Perm_T ps) {
  Perm_T p;

  ASSERT(ps);

  NEW(p);
  p->perm       = ps->perm;
  p->action     = ps->action;
  current->perm = p;
  reset_permset();

}

/*
 * Set Match object in the current service
 */
static void addmatch(Match_T ms, int actionnumber, int linenumber) {
  Match_T m;
  Match_T ml;
  int     reg_return;
  
  ASSERT(ms);

  NEW(m);
#ifdef HAVE_REGEX_H
  NEW(m->regex_comp);
#endif

  m->match_string = ms->match_string;
  m->match_path   = ms->match_path ? xstrdup(ms->match_path) : NULL;
  m->action       = ms->action;
  m->not          = ms->not;
  m->ignore       = ms->ignore;
  m->next         = NULL;

  addeventaction(&(m->action), actionnumber, ACTION_IGNORE);

#ifdef HAVE_REGEX_H
  reg_return = regcomp(m->regex_comp, ms->match_string, REG_NOSUB|REG_EXTENDED);

  if (reg_return != 0) {
    char errbuf[STRLEN];
    regerror(reg_return, ms->regex_comp, errbuf, STRLEN);
    if (m->match_path != NULL) 
      yyerror2("regex parsing error:%s on line %i of", errbuf, linenumber);
    else
      yyerror2("regex parsing error:%s", errbuf);
  }
#endif

  if (current->matchlist) {
    /* Find the end of the list */
    for (ml = current->matchlist; ml->next; ml = ml->next)
      ;

    ml->next = m;
    
  } else
    current->matchlist = m;
}


static void addmatchpath(Match_T ms, int actionnumber) {

  FILE *handle;
  Command_T savecommand = NULL;
  char buf[2048];
  int linenumber = 0;

  ASSERT(ms->match_path);

  handle = fopen(ms->match_path, "r");
  if (handle == NULL) {
    yyerror2("cannot read regex match file (%s)", ms->match_path);
    return;
  }
  
  while (!feof(handle)) {
    int len;

    linenumber++;
    
    if (! fgets(buf, 2048, handle))
      continue;

    len = strlen(buf);

    if (len == 0 || buf[0] == '\n')
      continue;

    if (buf[len-1] == '\n')
      buf[len-1] = 0;

    ms->match_string = xstrdup(buf);

    /* The addeventaction() called from addmatch() will reset the
     * command1 to NULL, but we need to duplicate the command for
     * each line, thus need to save it here */
    if (actionnumber == ACTION_EXEC) {
      if (command1 == NULL) {
        ASSERT(savecommand);
        command1 = savecommand;
      }
      savecommand = copycommand(command1);
    }

    addmatch(ms, actionnumber, linenumber);
  }

  if (actionnumber == ACTION_EXEC && savecommand)
    gccmd(&savecommand);

  fclose(handle);
}


/*
 * Set Uid object in the current service
 */
static void adduid(Uid_T us) {
  Uid_T u;

  ASSERT(us);

  NEW(u);
  u->uid       = us->uid;
  u->action    = us->action;
  current->uid = u;
  reset_uidset();
}


/*
 * Set Gid object in the current service
 */
static void addgid(Gid_T gs) {
  Gid_T g;

  ASSERT(gs);

  NEW(g);
  g->gid       = gs->gid;
  g->action    = gs->action;
  current->gid = g;
  reset_gidset();
}


/*
 * Add a new filesystem to the current service's filesystem list
 */
static void addfilesystem(Filesystem_T ds) {
  Filesystem_T dev;

  ASSERT(ds);
  
  NEW(dev);
  dev->resource           = ds->resource;
  dev->operator           = ds->operator;
  dev->limit_absolute     = ds->limit_absolute;
  dev->limit_percent      = ds->limit_percent;
  dev->action             = ds->action;

  dev->next               = current->filesystemlist;
  current->filesystemlist = dev;

  reset_filesystemset();

}


/*
 * Add a new icmp object to the current service's icmp list
 */
static void addicmp(Icmp_T is) {
  Icmp_T icmp;

  ASSERT(is);

  NEW(icmp);
  icmp->type         = is->type;      
  icmp->count        = is->count;
  icmp->timeout      = is->timeout;
  icmp->action       = is->action;
  icmp->is_available = FALSE;
  icmp->response     = -1;
  
  icmp->next         = current->icmplist;
  current->icmplist  = icmp;

  reset_icmpset();
}


/*
 * Set EventAction object
 */
static void addeventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea;

  ASSERT(_ea);

  NEW(ea);
  NEW(ea->failed);
  NEW(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
  *_ea = ea;
  reset_rateset();
}


/*
 * Redefine EventAction object (used for default action overloading)
 */
static void seteventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea = *_ea;

  ASSERT(ea);
  ASSERT(ea->failed);
  ASSERT(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
}


/*
 * Return a protocol object for the given protocol
 */
static void *addprotocol(int protocol) {

  switch (protocol) {
  case P_APACHESTATUS:  return create_apache_status();
  case P_DNS:           return create_dns();
  case P_DWP:           return create_dwp();
  case P_FTP:           return create_ftp();
  case P_GENERIC:       return create_generic();
  case P_HTTP:          return create_http();
  case P_IMAP:          return create_imap();
  case P_CLAMAV:        return create_clamav();
  case P_LDAP2:         return create_ldap2();
  case P_LDAP3:         return create_ldap3();
  case P_MYSQL:         return create_mysql();
  case P_NNTP:          return create_nntp();
  case P_NTP3:          return create_ntp3();
  case P_POSTFIXPOLICY: return create_postfix_policy();
  case P_POP:           return create_pop();
  case P_SMTP:          return create_smtp();
  case P_SSH:           return create_ssh();
  case P_RDATE:         return create_rdate();
  case P_RSYNC:         return create_rsync();
  case P_TNS:           return create_tns();
  case P_PGSQL:         return create_pgsql();
  case P_SIP:           return create_sip();
  case P_LMTP:          return create_lmtp();
  case P_GPS:           return create_gps();
  case P_RADIUS:        return create_radius();
  case P_MEMCACHE:      return create_memcache();
  }

  return create_default();
   
}


/*
 * Add a generic protocol handler to 
 */
static void addgeneric(Port_T port, char *send, char *expect) {
  Generic_T g = port->generic;
  
  if (g == NULL) {
    NEW(g);
    port->generic = g;
  } else {
    while (g->next != NULL)
      g = g->next;
    NEW(g->next);
    g = g->next;
  }
  
  if (send != NULL) {
    g->send = xstrdup(send);
    g->expect = NULL;
  } else if (expect != NULL) {
#ifdef HAVE_REGEX_H
    
    int   reg_return;
    NEW(g->expect);
    reg_return = regcomp(g->expect, expect, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, g->expect, errbuf, STRLEN);
      yyerror2("regex parsing error:%s", errbuf);
    }
#else
    g->expect = xstrdup(expect);
#endif
    g->send = NULL;
  } 
}


/*
 * Add the current command object to the current service object's
 * start or stop program.
 */
static void addcommand(int what, unsigned timeout) {

  switch(what) {
  case START: current->start = command; break;
  case STOP:  current->stop = command; break;
  }

  command->timeout = timeout;
  
  command = NULL;
  
}

  
/*
 * Add a new argument to the argument list
 */
static void addargument(char *argument) {

  ASSERT(argument);

  if (! command) {
    
    NEW(command);
    check_exec(argument);
    
  }
  
  command->arg[command->length++] = argument;
  command->arg[command->length] = NULL;
  
  if (command->length >= ARGMAX)
    yyerror("exceeded maximum number of program arguments");
  
}


/*
 * Setup a url request for the current port object
 */
static void prepare_urlrequest(URL_T U) {

  ASSERT(U);
  
  portset.protocol = addprotocol(P_HTTP);

  if (urlrequest == NULL)
    NEW(urlrequest);
  urlrequest->url = U;
  portset.hostname = xstrdup(U->hostname);
  check_hostname(portset.hostname);
  portset.port = U->port;
  portset.url_request = urlrequest;
  portset.type = SOCK_STREAM;
  portset.request = Util_getString("%s%s%s", U->path, U->query ? "?" : "", U->query ? U->query : "");
  /* Only the HTTP protocol is supported for URLs.
     See also the lexer if this is to be changed in
     the future */
  portset.protocol = addprotocol(P_HTTP);
  if (IS(U->protocol, "https"))
    portset.SSL.use_ssl = TRUE;
  
}


/*
 * Set the url request for a port
 */
static void  seturlrequest(int operator, char *regex) {
  
  ASSERT(regex);

  if (! urlrequest)
    NEW(urlrequest);
  urlrequest->operator = operator;
#ifdef HAVE_REGEX_H
  {    
    int reg_return;
    NEW(urlrequest->regex);
    reg_return = regcomp(urlrequest->regex, regex, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, urlrequest->regex, errbuf, STRLEN);
      yyerror2("regex parsing error: %s", errbuf);
    }
  }
#else
  urlrequest->regex = xstrdup(regex);
#endif

}


/*
 * Add a new data recipient server to the mmonit server list
 */
static void addmmonit(URL_T url, int timeout, int sslversion, char *certmd5) {
  Mmonit_T c;
  
  ASSERT(url);

  NEW(c);
  c->url = url;
  if (!strcmp(c->url->protocol, "https")) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      c->ssl.use_ssl = TRUE;
      c->ssl.version = (sslversion == SSL_VERSION_NONE) ? SSL_VERSION_AUTO : sslversion;
      if (certmd5) {
	c->ssl.certmd5 = certmd5;
	cleanup_hash_string(c->ssl.certmd5);
      }
    }
  }
  c->timeout = timeout;
  c->next = NULL;

  if (Run.mmonits) {
    Mmonit_T C;
    for (C = Run.mmonits; C->next; C = C->next)
      /* Empty */ ;
    C->next = c;
  } else {
    Run.mmonits = c;
  }
}


/*
 * Add a new smtp server to the mail server list
 */
static void addmailserver(MailServer_T mailserver) {

  MailServer_T s;
  
  ASSERT(mailserver->host);

  NEW(s);
  s->host        = mailserver->host;
  s->port        = mailserver->port;
  s->username    = mailserver->username;
  s->password    = mailserver->password;
  s->ssl.use_ssl = mailserver->ssl.use_ssl;
  s->ssl.version = mailserver->ssl.version;
  s->ssl.certmd5 = mailserver->ssl.certmd5;

  s->next = NULL;

  if (Run.mailservers) {
    MailServer_T l;
    for (l = Run.mailservers; l->next; l = l->next) /* empty */;
    l->next = s;
  } else {
    Run.mailservers = s;
  }
  
  reset_mailserverset();
}


/*
 * Return uid if found on the system. If the parameter user is NULL
 * the uid parameter is used for looking up the user id on the system,
 * otherwise the user parameter is used.
 */
static uid_t get_uid(char *user, uid_t uid) {
  struct passwd *pwd;

  if (user) {
    pwd = getpwnam(user);

    if (pwd == NULL) {
      yyerror2("requested user not found on the system");
      return(0);
    }

  } else {

    if ( (pwd = getpwuid(uid)) == NULL ) {
      yyerror2("requested uid not found on the system");
      return(0);
    }
  }

  return(pwd->pw_uid);

}


/*
 * Return gid if found on the system. If the parameter group is NULL
 * the gid parameter is used for looking up the group id on the system,
 * otherwise the group parameter is used.
 */
static gid_t get_gid(char *group, gid_t gid) {
  struct group *grd;

  if (group) {
    grd = getgrnam(group);

    if (grd == NULL) {
      yyerror2("requested group not found on the system");
      return(0);
    }

  } else {

    if ( (grd = getgrgid(gid)) == NULL ) {
      yyerror2("requested gid not found on the system");
      return(0);
    }

  }

  return(grd->gr_gid);

}


/*
 * Add a new user id to the current command object.
 */
static void addeuid(uid_t uid) {
  if (!getuid()) {
    command->has_uid = TRUE;
    command->uid = uid;
  } else
    yyerror("uid statement requires root privileges");
}


/*
 * Add a new group id to the current command object.
 */
static void addegid(gid_t gid) {
  if (!getuid()) {
    command->has_gid = TRUE;
    command->gid = gid;
  } else
    yyerror("gid statement requires root privileges");
}


/*
 * Reset the logfile if changed
 */
static void setlogfile(char *logfile) {
  if (Run.logfile) {
    if (IS(Run.logfile, logfile)) {
      FREE(logfile);
      return;
    } else
      FREE(Run.logfile);
  }
  Run.logfile = logfile;
}


/*
 * Reset the pidfile if changed
 */
static void setpidfile(char *pidfile) {
  if (Run.pidfile) {
    if (IS(Run.pidfile, pidfile)) {
      FREE(pidfile);
      return;
    } else
      FREE(Run.pidfile);
  }
  Run.pidfile = pidfile;
}


/*
 * Read a apache htpasswd file and add credentials found for username
 */
static void addhtpasswdentry(char *filename, char *username, int dtype) {
  char *ht_username = NULL;
  char *ht_passwd = NULL;
  char buf[STRLEN];
  FILE *handle = NULL;
  int credentials_added = 0;
  
  ASSERT(filename);

  handle = fopen(filename, "r");

  if ( handle == NULL ) {
    if (username != NULL)
      yyerror2("cannot read htpasswd (%s)", filename);
    else
      yyerror2("cannot read htpasswd", filename);
    return;
  }
  
  while (!feof(handle)) {
    char *colonindex = NULL;
    int i;
    
    if (! fgets(buf, STRLEN, handle))
      continue;

    /* strip trailing non visible characters */
    for (i = strlen(buf)-1; i >= 0; i--) {
      if ( buf[i] == ' '  || buf[i] == '\r' || buf[i] == '\n' || buf[i] == '\t' )
        buf[i] ='\0';
      else
        break;
    }

    if ( NULL == (colonindex = strchr(buf, ':')))
      continue;

    ht_passwd = xstrdup(colonindex+1);
    *colonindex = '\0';

    /* In case we have a file in /etc/passwd or /etc/shadow style we
     *  want to remove ":.*$" and Crypt and MD5 hashed dont have a colon
     */ 
    
    if ( (NULL != (colonindex = strchr(ht_passwd, ':'))) && ( dtype != DIGEST_CLEARTEXT) )
      *colonindex = '\0';

    ht_username = xstrdup(buf);

    if (username == NULL) {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else if (strcmp(username, ht_username) == 0)  {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else {
      FREE(ht_passwd);
      FREE(ht_username);
    }
  }

  if (credentials_added == 0) {
    if ( username == NULL )
      yywarning2("htpasswd file (%s) has no usable credentials", filename);
    else
      yywarning2("htpasswd file (%s) has no usable credentials for user %s", filename, username);
  }
  fclose(handle);
}


#ifdef HAVE_LIBPAM
static void addpamauth(char* groupname, int readonly) {
  Auth_T c = NULL;
  Auth_T prev = NULL;

  ASSERT(groupname);

  if (Run.credentials == NULL)
    NEW(Run.credentials);

  c = Run.credentials;
  do {
    if (c->groupname != NULL && IS(c->groupname, groupname)) {
      yywarning2("PAM group %s was added already, entry ignored", groupname);
      FREE(groupname);
      return;
    }
    prev = c;
    c = c->next;
  } while (c != NULL);

  NEW(prev->next);
  c = prev->next;

  c->next        = NULL;
  c->uname       = NULL;
  c->passwd      = NULL;
  c->groupname   = groupname;
  c->digesttype  = DIGEST_PAM;
  c->is_readonly = readonly;
  
  DEBUG("%s: Adding PAM group '%s'.\n", prog, groupname); 

  return;
}
#endif


/*
 * Add Basic Authentication credentials
 */
static int addcredentials(char *uname, char *passwd, int dtype, int readonly) {
  Auth_T c;

  ASSERT(uname);
  ASSERT(passwd);

  if (Run.credentials == NULL) {
    NEW(Run.credentials);
    c = Run.credentials;
  } else {

    if (Util_getUserCredentials(uname) != NULL) {
      yywarning2("credentials for user %s were already added, entry ignored", uname);
      FREE(uname);
      FREE(passwd);
      return FALSE;
    }

    c = Run.credentials;

    while ( c->next != NULL )
      c = c->next;

    NEW(c->next);
    c = c->next;
        
  }
  
  c->next        = NULL;
  c->uname       = uname;
  c->passwd      = passwd;
  c->groupname   = NULL;
  c->digesttype  = dtype;
  c->is_readonly = readonly;
  
  DEBUG("%s: Debug: Adding credentials for user '%s'.\n", prog, uname); 
  
  return TRUE;
  
}


/*
 * Set the syslog and the facilities to be used
 */
static void setsyslog(char *facility) {

  if (!Run.logfile || ihp.logfile) {
    ihp.logfile = TRUE;
    setlogfile(xstrdup("syslog"));
    Run.use_syslog = TRUE;
    Run.dolog = TRUE;
  }

  if (facility) {
    if (IS(facility,"log_local0"))
      Run.facility = LOG_LOCAL0;
    else if (IS(facility, "log_local1"))
      Run.facility = LOG_LOCAL1;
    else if (IS(facility, "log_local2"))
      Run.facility = LOG_LOCAL2;
    else if (IS(facility, "log_local3"))
      Run.facility = LOG_LOCAL3;
    else if (IS(facility, "log_local4"))
      Run.facility = LOG_LOCAL4;
    else if (IS(facility, "log_local5"))
      Run.facility = LOG_LOCAL5;
    else if (IS(facility, "log_local6"))
      Run.facility = LOG_LOCAL6;
    else if (IS(facility, "log_local7"))
      Run.facility = LOG_LOCAL7;
    else if (IS(facility, "log_daemon"))
      Run.facility = LOG_DAEMON;
    else
      yyerror2("invalid syslog facility");
  } else {
    Run.facility = LOG_USER;
  }
  
}


/*
 * Reset the current mailset, eventset and reminder for reuse
 */
static void reset_mailset() {
  memset(&mailset, 0, sizeof(struct mymail));
  eventset = Event_Null;
}


/*
 * Reset the mailserver set to default values
 */
static void reset_mailserverset() {
  memset(&mailserverset, 0, sizeof(struct mymailserver));
  mailserverset.port = PORT_SMTP;
  mailserverset.ssl.use_ssl = FALSE;
  mailserverset.ssl.version = SSL_VERSION_AUTO;
}


/*
 * Reset the Port set to default values
 */
static void reset_portset() {
  memset(&portset, 0, sizeof(struct myport));
  portset.socket = -1;
  portset.type = SOCK_STREAM;
  portset.family = AF_INET;
  portset.SSL.version = SSL_VERSION_AUTO;
  portset.timeout = NET_TIMEOUT;
  portset.maxforward = 70;
  urlrequest = NULL;
}


/*
 * Reset the Proc set to default values
 */
static void reset_resourceset() {
  resourceset.resource_id = 0;
  resourceset.limit = 0;
  resourceset.action = NULL;
  resourceset.operator = OPERATOR_EQUAL;
}


/*
 * Reset the Timestamp set to default values
 */
static void reset_timestampset() {
  timestampset.operator = OPERATOR_EQUAL;
  timestampset.time = 0;
  timestampset.test_changes = FALSE;
  timestampset.action = NULL;
}


/*
 * Reset the ActionRate set to default values
 */
static void reset_actionrateset() {
  actionrateset.count = 0;
  actionrateset.cycle = 0;
  actionrateset.action = NULL;
}


/*
 * Reset the Size set to default values
 */
static void reset_sizeset() {
  sizeset.operator = OPERATOR_EQUAL;
  sizeset.size = 0;
  sizeset.test_changes = FALSE;
  sizeset.action = NULL;
}


/*
 * Reset the Checksum set to default values
 */
static void reset_checksumset() {
  checksumset.type         = HASH_UNKNOWN;
  checksumset.test_changes = FALSE;
  checksumset.action       = NULL;
  *checksumset.hash        = 0;
}


/*
 * Reset the Perm set to default values
 */
static void reset_permset() {
  permset.perm = 0;
  permset.action = NULL;
}


/*
 * Reset the Uid set to default values
 */
static void reset_uidset() {
  uidset.uid = 0;
  uidset.action = NULL;
}


/*
 * Reset the Gid set to default values
 */
static void reset_gidset() {
  gidset.gid = 0;
  gidset.action = NULL;
}


/*
 * Reset the Filesystem set to default values
 */
static void reset_filesystemset() {
  filesystemset.resource = 0;
  filesystemset.operator = OPERATOR_EQUAL;
  filesystemset.limit_absolute = -1;
  filesystemset.limit_percent = -1;
  filesystemset.action = NULL;
}


/*
 * Reset the ICMP set to default values
 */
static void reset_icmpset() {
  icmpset.type = ICMP_ECHO;
  icmpset.count = ICMP_ATTEMPT_COUNT;
  icmpset.timeout = NET_TIMEOUT;
  icmpset.action = NULL;
}


/*
 * Reset the Rate set to default values
 */
static void reset_rateset() {
  rate1.count  = 1;
  rate1.cycles = 1;

  rate2.count  = 1;
  rate2.cycles = 1;
}


/* ---------------------------------------------------------------- Checkers */


/*
 * Check for unique service name
 */
static void check_name(char *name) {
  ASSERT(name);

  if (Util_existService(name) || (current && IS(name, current->name)))
    yyerror2("service name conflict, %s already defined", name);
  if (name && *name == '/')		
          yyerror2("service name '%s' must not start with '/' -- ", name);	
}


/*
 * Permission statement semantic check
 */
static int check_perm(int perm) {
  long result;
  char *status;
  char buf[STRLEN];

  snprintf(buf, STRLEN, "%d", perm);

  result = strtol(buf, &status, 8);

  if ( *status != '\0' || result < 0 || result > 07777 )
    yyerror2("permission statements must have an octal value between 0 and 7777");

  return result;
}


/*
 * Every statement semantic check
 */
static void check_every(int every) {
  if (every <= 1)
    yyerror2("an EVERY statement must have a value greater than 1");
}


/*
 * Check hostname 
 */
static void check_hostname(char *hostname) {

  ASSERT(hostname);

  if (!check_host(hostname))
    yywarning2("hostname did not resolve");
}

/*
 * Check the dependency graph for errors
 * by doing a topological sort, thereby finding any cycles.
 * Assures that graph is a Directed Acyclic Graph (DAG).
 */
static void check_depend() {
  Service_T s;
  Service_T depends_on = NULL;
  Service_T* dlt = &depend_list; /* the current tail of it                                 */
  int done;                      /* no unvisited nodes left?                               */
  int found_some;                /* last iteration found anything new ?                    */
  depend_list = NULL;            /* depend_list will be the topological sorted servicelist */

  do { 
    done = TRUE;
    found_some = FALSE; 
    for (s = servicelist; s; s = s->next) {
        Dependant_T d;
      if (s->visited)
	  continue;
      done = FALSE; // still unvisited nodes
      depends_on = NULL;
      for (d = s->dependantlist; d; d = d->next) {
        Service_T dp = Util_getService(d->dependant);
        if (!dp) {
          LogError("%s: Error: Depend service '%s' is not defined in the control file\n", prog, d->dependant);
          exit(1);
        }
        if (!dp->visited) {
          depends_on = dp;
        }
      }

      if (!depends_on) {
        s->visited = TRUE;
        found_some = TRUE;
        *dlt = s;
        dlt = &s->next_depend;
      }
    }
  } while (found_some && !done);
	
  if (!done) {
        ASSERT(depends_on);
	LogError("%s: Error: Found a depend loop in the control file involving the service '%s'\n", prog, depends_on->name);
	exit(1);
   } 

  ASSERT(depend_list);
  servicelist = depend_list;
    
  for (s = depend_list; s; s = s->next_depend)
    s->next = s->next_depend;
    
  reset_depend();
}


/*
 * Check if the executable exist
 */
static void check_exec(char *exec) {
  if (! File_exist(exec))
    yywarning2("the executable does not exist");
}
 
 
/* Return a valid max forward value for SIP header */
static int verifyMaxForward(int mf) { 
  int max = 70;
  
  if (mf >= 0 && mf <= 255)
    max = mf;
  else
    yywarning2("SIP max forward is outside the range [0..255]. Setting max forward to 70");
  
  return max;
}


/* -------------------------------------------------------------------- Misc */


/*
 * Cleans up an md5 string, tolower and remove byte separators
 */
static int cleanup_hash_string(char *hashstring) {
  int i = 0, j = 0;

  ASSERT(hashstring);

  while (hashstring[i] != '\0') {
    if (isxdigit((int) hashstring[i])) {
      hashstring[j] = tolower((int)hashstring[i]);
      j++;
    } 
    i++;
  }
  hashstring[j] = '\0';
  return j;
}


/* Return deep copy of the command */
static Command_T copycommand(Command_T source) {
  int i;
  Command_T copy = NULL;

  NEW(copy);
  copy->length = source->length;
  copy->has_uid = source->has_uid;
  copy->uid = source->uid;
  copy->has_gid = source->has_gid;
  copy->gid = source->gid;
  copy->timeout = source->timeout;
  for (i = 0; i < copy->length; i++)
     copy->arg[i] = xstrdup(source->arg[i]);
  copy->arg[copy->length] = NULL;

  return copy;
}

#line 3694 "y.tab.c"

#if YYDEBUG
#include <stdio.h>		/* needed for printf */
#endif

#include <stdlib.h>	/* needed for malloc, etc */
#include <string.h>	/* needed for memset */

/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(YYSTACKDATA *data)
{
    int i;
    unsigned newsize;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = data->stacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = data->s_mark - data->s_base;
    newss = (short *)realloc(data->s_base, newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    data->s_base = newss;
    data->s_mark = newss + i;

    newvs = (YYSTYPE *)realloc(data->l_base, newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    data->l_base = newvs;
    data->l_mark = newvs + i;

    data->stacksize = newsize;
    data->s_last = data->s_base + newsize - 1;
    return 0;
}

#if YYPURE || defined(YY_NO_LEAKS)
static void yyfreestack(YYSTACKDATA *data)
{
    free(data->s_base);
    free(data->l_base);
    memset(data, 0, sizeof(*data));
}
#else
#define yyfreestack(data) /* nothing */
#endif

#define YYABORT  goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR  goto yyerrlab

int
YYPARSE_DECL()
{
    int yym, yyn, yystate;
#if YYDEBUG
    const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;
    yystate = 0;

#if YYPURE
    memset(&yystack, 0, sizeof(yystack));
#endif

    if (yystack.s_base == NULL && yygrowstack(&yystack)) goto yyoverflow;
    yystack.s_mark = yystack.s_base;
    yystack.l_mark = yystack.l_base;
    yystate = 0;
    *yystack.s_mark = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = YYLEX) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
        {
            goto yyoverflow;
        }
        yystate = yytable[yyn];
        *++yystack.s_mark = yytable[yyn];
        *++yystack.l_mark = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

    goto yyerrlab;

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yystack.s_mark]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yystack.s_mark, yytable[yyn]);
#endif
                if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
                {
                    goto yyoverflow;
                }
                yystate = yytable[yyn];
                *++yystack.s_mark = yytable[yyn];
                *++yystack.l_mark = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yystack.s_mark);
#endif
                if (yystack.s_mark <= yystack.s_base) goto yyabort;
                --yystack.s_mark;
                --yystack.l_mark;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yystack.l_mark[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 139:
#line 487 "p.y"
	{
                    addmail(yystack.l_mark[-5].string, &mailset, &Run.maillist, eventset, yystack.l_mark[0].number);
                  }
break;
case 140:
#line 490 "p.y"
	{
                    addmail(yystack.l_mark[-2].string, &mailset, &Run.maillist, Event_All, yystack.l_mark[0].number);
                  }
break;
case 141:
#line 493 "p.y"
	{
                   addmail(yystack.l_mark[-6].string, &mailset, &Run.maillist, ~eventset, yystack.l_mark[0].number);
                  }
break;
case 142:
#line 498 "p.y"
	{
                    if (!Run.isdaemon || ihp.daemon) {
                      ihp.daemon     = TRUE;
                      Run.isdaemon   = TRUE;
                      Run.polltime   = yystack.l_mark[-1].number;
                      Run.startdelay = yystack.l_mark[0].number;
                    }
                  }
break;
case 143:
#line 508 "p.y"
	{ yyval.number = START_DELAY; }
break;
case 144:
#line 509 "p.y"
	{ yyval.number = yystack.l_mark[0].number; }
break;
case 145:
#line 512 "p.y"
	{
                    Run.expectbuffer = yystack.l_mark[-1].number * yystack.l_mark[0].number;
                  }
break;
case 146:
#line 517 "p.y"
	{
                    Run.init = TRUE;
                  }
break;
case 147:
#line 522 "p.y"
	{
                  #ifdef OPENSSL_FIPS
                    Run.fipsEnabled = TRUE;
                  #endif
                  }
break;
case 148:
#line 529 "p.y"
	{
                   if (!Run.logfile || ihp.logfile) {
                     ihp.logfile = TRUE;
                     setlogfile(yystack.l_mark[0].string);
                     Run.use_syslog = FALSE;
                     Run.dolog =TRUE;
                   }
                  }
break;
case 149:
#line 537 "p.y"
	{
                    setsyslog(NULL);
                  }
break;
case 150:
#line 540 "p.y"
	{
                    setsyslog(yystack.l_mark[0].string); FREE(yystack.l_mark[0].string);
                  }
break;
case 151:
#line 545 "p.y"
	{
                    Run.eventlist_dir = yystack.l_mark[0].string;
                  }
break;
case 152:
#line 548 "p.y"
	{
                    Run.eventlist_dir = yystack.l_mark[-2].string;
                    Run.eventlist_slots = yystack.l_mark[0].number;
                  }
break;
case 153:
#line 552 "p.y"
	{
                    Run.eventlist_dir = xstrdup(MYEVENTLISTBASE);
                    Run.eventlist_slots = yystack.l_mark[0].number;
                  }
break;
case 154:
#line 558 "p.y"
	{
                    Run.idfile = yystack.l_mark[0].string;
                  }
break;
case 155:
#line 563 "p.y"
	{
                    Run.statefile = yystack.l_mark[0].string;
                  }
break;
case 156:
#line 568 "p.y"
	{
                   if (!Run.pidfile || ihp.pidfile) {
                     ihp.pidfile = TRUE;
                     setpidfile(yystack.l_mark[0].string);
                   }
                 }
break;
case 160:
#line 583 "p.y"
	{
                    check_hostname((yystack.l_mark[-3].url)->hostname);
                    addmmonit(yystack.l_mark[-3].url, yystack.l_mark[-2].number, yystack.l_mark[-1].number, yystack.l_mark[0].string); 
                  }
break;
case 162:
#line 590 "p.y"
	{
                    Run.dommonitcredentials = FALSE;
                  }
break;
case 163:
#line 595 "p.y"
	{
                   Run.mailserver_timeout = yystack.l_mark[-1].number;
                   Run.mail_hostname = yystack.l_mark[0].string;
                  }
break;
case 164:
#line 601 "p.y"
	{
                   Run.MailFormat.from    = mailset.from    ?  mailset.from    : xstrdup(ALERT_FROM);
                   Run.MailFormat.replyto = mailset.replyto ?  mailset.replyto : NULL;
                   Run.MailFormat.subject = mailset.subject ?  mailset.subject : xstrdup(ALERT_SUBJECT);
                   Run.MailFormat.message = mailset.message ?  mailset.message : xstrdup(ALERT_MESSAGE);
                   reset_mailset();
                 }
break;
case 165:
#line 610 "p.y"
	{
                   Run.dohttpd = TRUE;
                   Run.httpdport = yystack.l_mark[-1].number;
                 }
break;
case 168:
#line 620 "p.y"
	{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yystack.l_mark[-4].string);

                    check_hostname(yystack.l_mark[-4].string);
                    mailserverset.host = yystack.l_mark[-4].string;
                    mailserverset.username = yystack.l_mark[-3].string;
                    mailserverset.password = yystack.l_mark[-2].string;
                    mailserverset.ssl.version = yystack.l_mark[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      if (mailserverset.ssl.version == SSL_VERSION_SSLV2 ||
                         mailserverset.ssl.version == SSL_VERSION_SSLV3)
                         mailserverset.port = PORT_SMTPS;
                      mailserverset.ssl.certmd5 = yystack.l_mark[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 169:
#line 639 "p.y"
	{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yystack.l_mark[-6].string);

                    check_hostname(yystack.l_mark[-6].string);
                    mailserverset.host = yystack.l_mark[-6].string;
                    mailserverset.port = yystack.l_mark[-4].number;
                    mailserverset.username = yystack.l_mark[-3].string;
                    mailserverset.password = yystack.l_mark[-2].string;
                    mailserverset.ssl.version = yystack.l_mark[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      mailserverset.ssl.certmd5 = yystack.l_mark[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 176:
#line 668 "p.y"
	{ Run.httpdssl = FALSE; }
break;
case 177:
#line 669 "p.y"
	{ 
                    Run.httpdssl = TRUE;                   
                    if (!have_ssl())
                      yyerror("SSL is not supported");
                  }
break;
case 183:
#line 683 "p.y"
	{ FREE(yystack.l_mark[0].string); }
break;
case 184:
#line 684 "p.y"
	{ FREE(yystack.l_mark[0].string); }
break;
case 186:
#line 688 "p.y"
	{ Run.httpdsig = TRUE; }
break;
case 187:
#line 689 "p.y"
	{ Run.httpdsig = FALSE; }
break;
case 192:
#line 700 "p.y"
	{ Run.bind_addr = yystack.l_mark[0].string; }
break;
case 193:
#line 703 "p.y"
	{
                    Run.httpsslpem = yystack.l_mark[0].string;
                    if (!File_checkStat(Run.httpsslpem, "SSL server PEM file", S_IRWXU))
                      yyerror2("SSL server PEM file has too loose permissions");
                  }
break;
case 195:
#line 711 "p.y"
	{
                    Run.httpsslclientpem = yystack.l_mark[0].string; 
                    Run.clientssl = TRUE;
                    if (!File_checkStat(Run.httpsslclientpem, "SSL client PEM file", S_IRWXU | S_IRGRP | S_IROTH))
                      yyerror2("SSL client PEM file has too loose permissions");
                  }
break;
case 196:
#line 719 "p.y"
	{ 
                   Run.allowselfcert = FALSE; 
                   config_ssl(Run.allowselfcert); 
                 }
break;
case 197:
#line 723 "p.y"
	{   
                    Run.allowselfcert = TRUE;
                    config_ssl(Run.allowselfcert); 
                  }
break;
case 198:
#line 729 "p.y"
	{
                    addcredentials(yystack.l_mark[-3].string,yystack.l_mark[-1].string, DIGEST_CLEARTEXT, yystack.l_mark[0].number);
                  }
break;
case 199:
#line 732 "p.y"
	{
#ifdef HAVE_LIBPAM
                    addpamauth(yystack.l_mark[-1].string, yystack.l_mark[0].number);
#else
                    yyerror("PAM is not supported");
                    FREE(yystack.l_mark[-1].string);
#endif
                  }
break;
case 200:
#line 740 "p.y"
	{
                    addhtpasswdentry(yystack.l_mark[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 201:
#line 744 "p.y"
	{
                    addhtpasswdentry(yystack.l_mark[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 202:
#line 748 "p.y"
	{
                    addhtpasswdentry(yystack.l_mark[0].string, NULL, DIGEST_MD5);
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 203:
#line 752 "p.y"
	{
                    addhtpasswdentry(yystack.l_mark[0].string, NULL, DIGEST_CRYPT);
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 204:
#line 756 "p.y"
	{
                    htpasswd_file = yystack.l_mark[0].string;
                    digesttype = CLEARTEXT;
                  }
break;
case 205:
#line 760 "p.y"
	{
                    FREE(htpasswd_file);
                  }
break;
case 206:
#line 763 "p.y"
	{
                    htpasswd_file = yystack.l_mark[0].string;
                    digesttype = DIGEST_CLEARTEXT;
                  }
break;
case 207:
#line 767 "p.y"
	{
                    FREE(htpasswd_file);
                  }
break;
case 208:
#line 770 "p.y"
	{
                    htpasswd_file = yystack.l_mark[0].string;
                    digesttype = DIGEST_MD5;
                  }
break;
case 209:
#line 774 "p.y"
	{
                    FREE(htpasswd_file);
                  }
break;
case 210:
#line 777 "p.y"
	{
                    htpasswd_file = yystack.l_mark[0].string;
                    digesttype = DIGEST_CRYPT;
                  }
break;
case 211:
#line 781 "p.y"
	{
                    FREE(htpasswd_file);
                  }
break;
case 212:
#line 784 "p.y"
	{
                    if (! (add_net_allow(yystack.l_mark[0].string) || add_host_allow(yystack.l_mark[0].string))) {
                      yyerror2("erroneous network or host identifier %s", yystack.l_mark[0].string); 
                    }
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 215:
#line 796 "p.y"
	{ addhtpasswdentry(htpasswd_file, yystack.l_mark[0].string, digesttype);
                           FREE(yystack.l_mark[0].string); }
break;
case 216:
#line 800 "p.y"
	{ yyval.number = FALSE; }
break;
case 217:
#line 801 "p.y"
	{ yyval.number = TRUE; }
break;
case 218:
#line 804 "p.y"
	{
                    createservice(TYPE_PROCESS, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_process);
                  }
break;
case 219:
#line 807 "p.y"
	{
                    createservice(TYPE_PROCESS, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_process);
                  }
break;
case 220:
#line 810 "p.y"
	{
                    createservice(TYPE_PROCESS, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_process);
                    matchset.ignore = FALSE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yystack.l_mark[0].string);
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 221:
#line 817 "p.y"
	{
                    createservice(TYPE_PROCESS, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_process);
                    matchset.ignore = FALSE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yystack.l_mark[0].string);
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 222:
#line 826 "p.y"
	{
                    createservice(TYPE_FILE, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_file);
                  }
break;
case 223:
#line 831 "p.y"
	{
                    createservice(TYPE_FILESYSTEM, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_filesystem);
                  }
break;
case 224:
#line 836 "p.y"
	{
                    createservice(TYPE_DIRECTORY, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_directory);
                  }
break;
case 225:
#line 841 "p.y"
	{
                    check_hostname(yystack.l_mark[0].string); 
                    createservice(TYPE_HOST, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_remote_host);
                  }
break;
case 226:
#line 847 "p.y"
	{
                    createservice(TYPE_SYSTEM, yystack.l_mark[0].string, xstrdup(""), check_system);
                    hassystem = TRUE;
                  }
break;
case 227:
#line 853 "p.y"
	{
                    createservice(TYPE_FIFO, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_fifo);
                  }
break;
case 228:
#line 858 "p.y"
	{
                    createservice(TYPE_STATUS, yystack.l_mark[-2].string, yystack.l_mark[0].string, check_status);
                  }
break;
case 229:
#line 863 "p.y"
	{
                    addcommand(START, yystack.l_mark[0].number);
                  }
break;
case 230:
#line 866 "p.y"
	{
                    addcommand(START, yystack.l_mark[0].number);
                  }
break;
case 231:
#line 871 "p.y"
	{
                    addcommand(STOP, yystack.l_mark[0].number);
                  }
break;
case 232:
#line 874 "p.y"
	{
                    addcommand(STOP, yystack.l_mark[0].number);
                  }
break;
case 237:
#line 887 "p.y"
	{ addargument(yystack.l_mark[0].string); }
break;
case 238:
#line 888 "p.y"
	{ addargument(yystack.l_mark[0].string); }
break;
case 239:
#line 891 "p.y"
	{ addeuid( get_uid(yystack.l_mark[0].string, 0) ); FREE(yystack.l_mark[0].string); }
break;
case 240:
#line 892 "p.y"
	{ addegid( get_gid(yystack.l_mark[0].string, 0) ); FREE(yystack.l_mark[0].string); }
break;
case 241:
#line 893 "p.y"
	{ addeuid( get_uid(NULL, yystack.l_mark[0].number) ); }
break;
case 242:
#line 894 "p.y"
	{ addegid( get_gid(NULL, yystack.l_mark[0].number) ); }
break;
case 243:
#line 897 "p.y"
	{ yyval.string = NULL; }
break;
case 244:
#line 898 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 245:
#line 899 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 246:
#line 902 "p.y"
	{ yyval.string = NULL; }
break;
case 247:
#line 903 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 248:
#line 906 "p.y"
	{ yyval.string = NULL; }
break;
case 249:
#line 907 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 250:
#line 911 "p.y"
	{
                    portset.timeout = yystack.l_mark[-4].number;
                    addeventaction(&(portset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addport(&portset);
                  }
break;
case 251:
#line 917 "p.y"
	{ 
                    prepare_urlrequest(yystack.l_mark[-6].url);
                    portset.timeout = yystack.l_mark[-4].number;
                    addeventaction(&(portset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addport(&portset);
                  }
break;
case 252:
#line 926 "p.y"
	{
                   portset.timeout = yystack.l_mark[-4].number;
                   addeventaction(&(portset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                   addport(&portset);
                  }
break;
case 253:
#line 934 "p.y"
	{
                   icmpset.type = yystack.l_mark[-6].number;
                   icmpset.count = yystack.l_mark[-5].number;
                   icmpset.timeout = yystack.l_mark[-4].number;
                   addeventaction(&(icmpset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                   addicmp(&icmpset);
                  }
break;
case 254:
#line 943 "p.y"
	{
                    if (current->type == TYPE_HOST)
                      portset.hostname = xstrdup(current->path);
                    else
                      portset.hostname = xstrdup(LOCALHOST);
                  }
break;
case 255:
#line 949 "p.y"
	{ check_hostname(yystack.l_mark[0].string); portset.hostname = yystack.l_mark[0].string; }
break;
case 256:
#line 952 "p.y"
	{ portset.port = yystack.l_mark[0].number; portset.family = AF_INET; }
break;
case 257:
#line 955 "p.y"
	{
                    portset.pathname = yystack.l_mark[0].string; portset.family = AF_UNIX;
                  }
break;
case 258:
#line 960 "p.y"
	{
                    portset.type = SOCK_STREAM;
                  }
break;
case 259:
#line 963 "p.y"
	{
                    portset.type = SOCK_STREAM;
                  }
break;
case 260:
#line 966 "p.y"
	{
                    portset.type = SOCK_STREAM;
                    portset.SSL.use_ssl = TRUE;
                    portset.SSL.version = yystack.l_mark[-1].number;
                    if (portset.SSL.version == SSL_VERSION_NONE)
                      portset.SSL.version = SSL_VERSION_AUTO;
                    portset.SSL.certmd5 = yystack.l_mark[0].string;
                  }
break;
case 261:
#line 974 "p.y"
	{
                    portset.type = SOCK_DGRAM;
                  }
break;
case 262:
#line 979 "p.y"
	{ yyval.string = NULL; }
break;
case 263:
#line 980 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 264:
#line 983 "p.y"
	{ yyval.number = SSL_VERSION_NONE; }
break;
case 265:
#line 984 "p.y"
	{ yyval.number = SSL_VERSION_SSLV2; }
break;
case 266:
#line 985 "p.y"
	{ yyval.number = SSL_VERSION_SSLV3; }
break;
case 267:
#line 986 "p.y"
	{ yyval.number = SSL_VERSION_TLS; }
break;
case 268:
#line 987 "p.y"
	{ yyval.number = SSL_VERSION_AUTO; }
break;
case 269:
#line 990 "p.y"
	{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 270:
#line 993 "p.y"
	{
                    portset.protocol = addprotocol(P_APACHESTATUS);
                  }
break;
case 271:
#line 996 "p.y"
	{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 272:
#line 999 "p.y"
	{
                    portset.protocol = addprotocol(P_DNS);
                  }
break;
case 273:
#line 1002 "p.y"
	{
                    portset.protocol = addprotocol(P_DWP);
                  }
break;
case 274:
#line 1005 "p.y"
	{
                    portset.protocol = addprotocol(P_FTP);
                  }
break;
case 275:
#line 1008 "p.y"
	{
                    portset.protocol = addprotocol(P_HTTP);
                  }
break;
case 276:
#line 1011 "p.y"
	{
                    portset.protocol = addprotocol(P_IMAP);
                  }
break;
case 277:
#line 1014 "p.y"
	{
                    portset.protocol = addprotocol(P_CLAMAV);
                  }
break;
case 278:
#line 1017 "p.y"
	{
                    portset.protocol = addprotocol(P_LDAP2);
                  }
break;
case 279:
#line 1020 "p.y"
	{
                    portset.protocol = addprotocol(P_LDAP3);
                  }
break;
case 280:
#line 1023 "p.y"
	{
                    portset.protocol = addprotocol(P_MYSQL);
                  }
break;
case 281:
#line 1026 "p.y"
	{
                    portset.protocol = addprotocol(P_SIP);
                  }
break;
case 282:
#line 1029 "p.y"
	{
                    portset.protocol = addprotocol(P_NNTP);
                  }
break;
case 283:
#line 1032 "p.y"
	{
                    portset.protocol = addprotocol(P_NTP3);
                    portset.type = SOCK_DGRAM;
                  }
break;
case 284:
#line 1036 "p.y"
	{
                    portset.protocol = addprotocol(P_POSTFIXPOLICY);
                  }
break;
case 285:
#line 1039 "p.y"
	{
                    portset.protocol = addprotocol(P_POP);
                  }
break;
case 286:
#line 1042 "p.y"
	{
                    portset.protocol = addprotocol(P_SMTP);
                  }
break;
case 287:
#line 1045 "p.y"
	{
                    portset.protocol = addprotocol(P_SSH);
                  }
break;
case 288:
#line 1048 "p.y"
	{
                    portset.protocol = addprotocol(P_RDATE);
                  }
break;
case 289:
#line 1051 "p.y"
	{
                    portset.protocol = addprotocol(P_RSYNC);
                  }
break;
case 290:
#line 1054 "p.y"
	{
                    portset.protocol = addprotocol(P_TNS);
                  }
break;
case 291:
#line 1057 "p.y"
	{
                    portset.protocol = addprotocol(P_PGSQL);
                  }
break;
case 292:
#line 1060 "p.y"
	{
                    portset.protocol = addprotocol(P_LMTP);
                  }
break;
case 293:
#line 1063 "p.y"
	{
                    portset.protocol = addprotocol(P_GPS);
                  }
break;
case 294:
#line 1066 "p.y"
	{
                    portset.protocol = addprotocol(P_RADIUS);
                  }
break;
case 295:
#line 1069 "p.y"
	{
                    portset.protocol = addprotocol(P_MEMCACHE);
                  }
break;
case 296:
#line 1072 "p.y"
	{
                    portset.protocol = addprotocol(P_GENERIC);
                  }
break;
case 299:
#line 1081 "p.y"
	{ addgeneric(&portset, yystack.l_mark[0].string, NULL); FREE(yystack.l_mark[0].string);}
break;
case 300:
#line 1082 "p.y"
	{ addgeneric(&portset, NULL, yystack.l_mark[0].string); FREE(yystack.l_mark[0].string);}
break;
case 302:
#line 1086 "p.y"
	{
                    portset.request = yystack.l_mark[0].string;
                  }
break;
case 303:
#line 1089 "p.y"
	{
                    portset.request = yystack.l_mark[0].string;
                  }
break;
case 305:
#line 1095 "p.y"
	{
                     portset.maxforward = verifyMaxForward(yystack.l_mark[0].number); 
                   }
break;
case 307:
#line 1101 "p.y"
	{ 
                    portset.request = Util_urlEncode(yystack.l_mark[-1].string); 
                    FREE(yystack.l_mark[-1].string); 
                  }
break;
case 308:
#line 1105 "p.y"
	{
                    portset.request = Util_urlEncode(yystack.l_mark[-3].string);
                    FREE(yystack.l_mark[-3].string);
                    portset.request_checksum = yystack.l_mark[-1].string;
                  }
break;
case 310:
#line 1113 "p.y"
	{
                    portset.request_hostheader = yystack.l_mark[0].string;
                  }
break;
case 311:
#line 1118 "p.y"
	{ 
                    portset.request = yystack.l_mark[0].string; 
                  }
break;
case 314:
#line 1127 "p.y"
	{ 
                    portset.ApacheStatus.loglimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.loglimit = (int)yystack.l_mark[-1].number; 
                  }
break;
case 315:
#line 1131 "p.y"
	{ 
                    portset.ApacheStatus.closelimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.closelimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 316:
#line 1135 "p.y"
	{ 
                    portset.ApacheStatus.dnslimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.dnslimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 317:
#line 1139 "p.y"
	{ 
                    portset.ApacheStatus.keepalivelimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.keepalivelimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 318:
#line 1143 "p.y"
	{ 
                    portset.ApacheStatus.replylimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.replylimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 319:
#line 1147 "p.y"
	{ 
                    portset.ApacheStatus.requestlimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.requestlimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 320:
#line 1151 "p.y"
	{ 
                    portset.ApacheStatus.startlimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.startlimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 321:
#line 1155 "p.y"
	{ 
                    portset.ApacheStatus.waitlimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.waitlimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 322:
#line 1159 "p.y"
	{ 
                    portset.ApacheStatus.gracefullimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.gracefullimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 323:
#line 1163 "p.y"
	{ 
                    portset.ApacheStatus.cleanuplimitOP = yystack.l_mark[-2].number; 
                    portset.ApacheStatus.cleanuplimit = (int)(yystack.l_mark[-1].number); 
                  }
break;
case 324:
#line 1169 "p.y"
	{
                    seteventaction(&(current)->action_NONEXIST, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                  }
break;
case 325:
#line 1175 "p.y"
	{
                    seteventaction(&(current)->action_PID, yystack.l_mark[0].number, ACTION_IGNORE);
                  }
break;
case 326:
#line 1180 "p.y"
	{
                    seteventaction(&(current)->action_PPID, yystack.l_mark[0].number, ACTION_IGNORE);
                  }
break;
case 327:
#line 1185 "p.y"
	{
                   yyval.number = ICMP_ATTEMPT_COUNT;
                  }
break;
case 328:
#line 1188 "p.y"
	{    
                   yyval.number = yystack.l_mark[0].number;
                  }
break;
case 329:
#line 1193 "p.y"
	{
                   yyval.number = EXEC_TIMEOUT;
                  }
break;
case 330:
#line 1196 "p.y"
	{
                   yyval.number = yystack.l_mark[-1].number;
                  }
break;
case 331:
#line 1201 "p.y"
	{
                   yyval.number = NET_TIMEOUT;
                  }
break;
case 332:
#line 1204 "p.y"
	{
                   yyval.number = yystack.l_mark[-1].number;
                  }
break;
case 333:
#line 1209 "p.y"
	{
                   actionrateset.count = yystack.l_mark[-5].number;
                   actionrateset.cycle = yystack.l_mark[-3].number;
                   addeventaction(&(actionrateset).action, yystack.l_mark[0].number, ACTION_IGNORE);
                   addactionrate(&actionrateset);
                 }
break;
case 334:
#line 1215 "p.y"
	{
                   actionrateset.count = yystack.l_mark[-5].number;
                   actionrateset.cycle = yystack.l_mark[-3].number;
                   addeventaction(&(actionrateset).action, ACTION_UNMONITOR, ACTION_IGNORE);
                   addactionrate(&actionrateset);
                 }
break;
case 336:
#line 1224 "p.y"
	{
                    seturlrequest(yystack.l_mark[-1].number, yystack.l_mark[0].string);
                    FREE(yystack.l_mark[0].string);
                  }
break;
case 337:
#line 1230 "p.y"
	{ yyval.number = OPERATOR_EQUAL; }
break;
case 338:
#line 1231 "p.y"
	{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 339:
#line 1234 "p.y"
	{
                   addmail(yystack.l_mark[-5].string, &mailset, &current->maillist, eventset, yystack.l_mark[0].number);
                  }
break;
case 340:
#line 1237 "p.y"
	{
                   addmail(yystack.l_mark[-2].string, &mailset, &current->maillist, Event_All, yystack.l_mark[0].number);
                  }
break;
case 341:
#line 1240 "p.y"
	{
                   addmail(yystack.l_mark[-6].string, &mailset, &current->maillist, ~eventset, yystack.l_mark[0].number);
                  }
break;
case 342:
#line 1243 "p.y"
	{
                   addmail(yystack.l_mark[0].string, &mailset, &current->maillist, Event_Null, 0);
                  }
break;
case 343:
#line 1248 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 344:
#line 1251 "p.y"
	{ yyval.string = yystack.l_mark[0].string; }
break;
case 347:
#line 1258 "p.y"
	{ eventset |= Event_Action; }
break;
case 348:
#line 1259 "p.y"
	{ eventset |= Event_Checksum; }
break;
case 349:
#line 1260 "p.y"
	{ eventset |= Event_Connection; }
break;
case 350:
#line 1261 "p.y"
	{ eventset |= Event_Content; }
break;
case 351:
#line 1262 "p.y"
	{ eventset |= Event_Data; }
break;
case 352:
#line 1263 "p.y"
	{ eventset |= Event_Exec; }
break;
case 353:
#line 1264 "p.y"
	{ eventset |= Event_Fsflag; }
break;
case 354:
#line 1265 "p.y"
	{ eventset |= Event_Gid; }
break;
case 355:
#line 1266 "p.y"
	{ eventset |= Event_Icmp; }
break;
case 356:
#line 1267 "p.y"
	{ eventset |= Event_Instance; }
break;
case 357:
#line 1268 "p.y"
	{ eventset |= Event_Invalid; }
break;
case 358:
#line 1269 "p.y"
	{ eventset |= Event_Nonexist; }
break;
case 359:
#line 1270 "p.y"
	{ eventset |= Event_Permission; }
break;
case 360:
#line 1271 "p.y"
	{ eventset |= Event_Pid; }
break;
case 361:
#line 1272 "p.y"
	{ eventset |= Event_PPid; }
break;
case 362:
#line 1273 "p.y"
	{ eventset |= Event_Resource; }
break;
case 363:
#line 1274 "p.y"
	{ eventset |= Event_Size; }
break;
case 364:
#line 1275 "p.y"
	{ eventset |= Event_Timeout; }
break;
case 365:
#line 1276 "p.y"
	{ eventset |= Event_Timestamp; }
break;
case 366:
#line 1277 "p.y"
	{ eventset |= Event_Uid; }
break;
case 371:
#line 1288 "p.y"
	{ mailset.from = yystack.l_mark[0].string; }
break;
case 372:
#line 1289 "p.y"
	{ mailset.replyto = yystack.l_mark[0].string; }
break;
case 373:
#line 1290 "p.y"
	{ mailset.subject = yystack.l_mark[0].string; }
break;
case 374:
#line 1291 "p.y"
	{ mailset.message = yystack.l_mark[0].string; }
break;
case 375:
#line 1294 "p.y"
	{
                   check_every(yystack.l_mark[-1].number);
                   current->def_every = TRUE;
                   current->every = yystack.l_mark[-1].number;
                 }
break;
case 376:
#line 1301 "p.y"
	{
                    current->mode = MODE_ACTIVE;
                  }
break;
case 377:
#line 1304 "p.y"
	{
                    current->mode = MODE_PASSIVE;
                  }
break;
case 378:
#line 1307 "p.y"
	{
                    current->mode = MODE_MANUAL;
                    current->monitor = MONITOR_NOT;
                  }
break;
case 379:
#line 1313 "p.y"
	{ addservicegroup(yystack.l_mark[0].string); FREE(yystack.l_mark[0].string);}
break;
case 383:
#line 1324 "p.y"
	{ adddependant(yystack.l_mark[0].string); }
break;
case 384:
#line 1327 "p.y"
	{
                     addeventaction(&(resourceset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                     addresource(&resourceset);
                   }
break;
case 391:
#line 1343 "p.y"
	{
                     addeventaction(&(resourceset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                     addresource(&resourceset);
                   }
break;
case 398:
#line 1359 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_CPU_PERCENT;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10); 
                  }
break;
case 399:
#line 1364 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_CPU_PERCENT;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10); 
                  }
break;
case 400:
#line 1371 "p.y"
	{
                    resourceset.resource_id = yystack.l_mark[-3].number;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10); 
                  }
break;
case 401:
#line 1378 "p.y"
	{ yyval.number = RESOURCE_ID_CPUUSER; }
break;
case 402:
#line 1379 "p.y"
	{ yyval.number = RESOURCE_ID_CPUSYSTEM; }
break;
case 403:
#line 1380 "p.y"
	{ yyval.number = RESOURCE_ID_CPUWAIT; }
break;
case 404:
#line 1383 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_MEM_KBYTE;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (int) (yystack.l_mark[-1].real * (yystack.l_mark[0].number / 1024.0)); 
                  }
break;
case 405:
#line 1388 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_MEM_PERCENT;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10); 
                  }
break;
case 406:
#line 1393 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_KBYTE;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (int) (yystack.l_mark[-1].real * (yystack.l_mark[0].number / 1024.0));
                  }
break;
case 407:
#line 1398 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_PERCENT;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10);
                  }
break;
case 408:
#line 1405 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_SWAP_KBYTE;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (int) (yystack.l_mark[-1].real * (yystack.l_mark[0].number / 1024.0));
                  }
break;
case 409:
#line 1410 "p.y"
	{
                    resourceset.resource_id = RESOURCE_ID_SWAP_PERCENT;
                    resourceset.operator = yystack.l_mark[-2].number;
                    resourceset.limit = (yystack.l_mark[-1].number * 10);
                  }
break;
case 410:
#line 1417 "p.y"
	{ 
                    resourceset.resource_id = RESOURCE_ID_CHILDREN;
                    resourceset.operator = yystack.l_mark[-1].number;
                    resourceset.limit = (int) yystack.l_mark[0].number; 
                  }
break;
case 411:
#line 1424 "p.y"
	{ 
                    resourceset.resource_id = yystack.l_mark[-2].number;
                    resourceset.operator = yystack.l_mark[-1].number;
                    resourceset.limit = (int) (yystack.l_mark[0].real * 10.0); 
                  }
break;
case 412:
#line 1431 "p.y"
	{ yyval.number = RESOURCE_ID_LOAD1; }
break;
case 413:
#line 1432 "p.y"
	{ yyval.number = RESOURCE_ID_LOAD5; }
break;
case 414:
#line 1433 "p.y"
	{ yyval.number = RESOURCE_ID_LOAD15; }
break;
case 415:
#line 1436 "p.y"
	{ yyval.real = yystack.l_mark[0].real; }
break;
case 416:
#line 1437 "p.y"
	{ yyval.real = (float) yystack.l_mark[0].number; }
break;
case 417:
#line 1440 "p.y"
	{
                    timestampset.operator = yystack.l_mark[-6].number;
                    timestampset.time = (yystack.l_mark[-5].number * yystack.l_mark[-4].number);
                    addeventaction(&(timestampset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addtimestamp(&timestampset, FALSE);
                  }
break;
case 418:
#line 1446 "p.y"
	{
                    timestampset.test_changes = TRUE;
                    addeventaction(&(timestampset).action, yystack.l_mark[0].number, ACTION_IGNORE);
                    addtimestamp(&timestampset, TRUE);
                  }
break;
case 419:
#line 1453 "p.y"
	{ yyval.number = OPERATOR_EQUAL; }
break;
case 420:
#line 1454 "p.y"
	{ yyval.number = OPERATOR_GREATER; }
break;
case 421:
#line 1455 "p.y"
	{ yyval.number = OPERATOR_LESS; }
break;
case 422:
#line 1456 "p.y"
	{ yyval.number = OPERATOR_EQUAL; }
break;
case 423:
#line 1457 "p.y"
	{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 424:
#line 1458 "p.y"
	{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 425:
#line 1461 "p.y"
	{ yyval.number = TIME_SECOND; }
break;
case 426:
#line 1462 "p.y"
	{ yyval.number = TIME_SECOND; }
break;
case 427:
#line 1463 "p.y"
	{ yyval.number = TIME_MINUTE; }
break;
case 428:
#line 1464 "p.y"
	{ yyval.number = TIME_HOUR; }
break;
case 429:
#line 1465 "p.y"
	{ yyval.number = TIME_DAY; }
break;
case 430:
#line 1468 "p.y"
	{ yyval.number = ACTION_ALERT; }
break;
case 431:
#line 1469 "p.y"
	{ yyval.number = ACTION_EXEC; }
break;
case 432:
#line 1470 "p.y"
	{ yyval.number = ACTION_EXEC; }
break;
case 433:
#line 1471 "p.y"
	{ yyval.number = ACTION_RESTART; }
break;
case 434:
#line 1472 "p.y"
	{ yyval.number = ACTION_START; }
break;
case 435:
#line 1473 "p.y"
	{ yyval.number = ACTION_STOP; }
break;
case 436:
#line 1474 "p.y"
	{ yyval.number = ACTION_UNMONITOR; }
break;
case 437:
#line 1477 "p.y"
	{
                    yyval.number = yystack.l_mark[0].number;
                    if (yystack.l_mark[0].number == ACTION_EXEC && command) {
                      command1 = command;
                      command = NULL;
                    }
                  }
break;
case 438:
#line 1486 "p.y"
	{
                    yyval.number = yystack.l_mark[0].number;
                    if (yystack.l_mark[0].number == ACTION_EXEC && command) {
                      command2 = command;
                      command = NULL;
                    }
                  }
break;
case 440:
#line 1496 "p.y"
	{
                    rate1.count  = yystack.l_mark[-1].number;
                    rate1.cycles = yystack.l_mark[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 441:
#line 1502 "p.y"
	{
                    rate1.count  = yystack.l_mark[-2].number;
                    rate1.cycles = yystack.l_mark[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate1.count < 1 || rate1.count > rate1.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 443:
#line 1513 "p.y"
	{
                    rate2.count  = yystack.l_mark[-1].number;
                    rate2.cycles = yystack.l_mark[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 444:
#line 1519 "p.y"
	{
                    rate2.count  = yystack.l_mark[-2].number;
                    rate2.cycles = yystack.l_mark[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate2.count < 1 || rate2.count > rate2.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 445:
#line 1529 "p.y"
	{
                    yyval.number = ACTION_ALERT;
                  }
break;
case 446:
#line 1532 "p.y"
	{
                    yyval.number = yystack.l_mark[0].number;
                  }
break;
case 447:
#line 1535 "p.y"
	{
                    yyval.number = yystack.l_mark[0].number;
                  }
break;
case 448:
#line 1538 "p.y"
	{
                    yyval.number = yystack.l_mark[0].number;
                  }
break;
case 449:
#line 1543 "p.y"
	{
                    addeventaction(&(checksumset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 450:
#line 1548 "p.y"
	{
                    snprintf(checksumset.hash, sizeof(checksumset.hash), "%s", yystack.l_mark[-4].string);
                    FREE(yystack.l_mark[-4].string);
                    addeventaction(&(checksumset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 451:
#line 1554 "p.y"
	{
                    checksumset.test_changes = TRUE;
                    addeventaction(&(checksumset).action, yystack.l_mark[0].number, ACTION_IGNORE);
                    addchecksum(&checksumset);
                  }
break;
case 452:
#line 1560 "p.y"
	{ checksumset.type = HASH_UNKNOWN; }
break;
case 453:
#line 1561 "p.y"
	{ checksumset.type = HASH_MD5; }
break;
case 454:
#line 1562 "p.y"
	{ checksumset.type = HASH_SHA1; }
break;
case 455:
#line 1565 "p.y"
	{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yystack.l_mark[-5].number;
                    filesystemset.limit_absolute = yystack.l_mark[-4].number;
                    addeventaction(&(filesystemset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 456:
#line 1572 "p.y"
	{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yystack.l_mark[-6].number;
                    filesystemset.limit_percent = (int)(yystack.l_mark[-5].number * 10);
                    addeventaction(&(filesystemset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 457:
#line 1581 "p.y"
	{
                    if (!filesystem_usage(current->inf, current->path))
                      yyerror2("cannot read usage of filesystem %s", current->path);
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yystack.l_mark[-6].number;
                    filesystemset.limit_absolute = (int)((float)yystack.l_mark[-5].real / (float)current->inf->f_bsize * (float)yystack.l_mark[-4].number);
                    addeventaction(&(filesystemset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 458:
#line 1590 "p.y"
	{
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yystack.l_mark[-6].number;
                    filesystemset.limit_percent = (int)(yystack.l_mark[-5].number * 10);
                    addeventaction(&(filesystemset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 459:
#line 1599 "p.y"
	{
                    seteventaction(&(current)->action_FSFLAG, yystack.l_mark[0].number, ACTION_IGNORE);
                  }
break;
case 460:
#line 1604 "p.y"
	{ yyval.number = UNIT_BYTE; }
break;
case 461:
#line 1605 "p.y"
	{ yyval.number = UNIT_BYTE; }
break;
case 462:
#line 1606 "p.y"
	{ yyval.number = UNIT_KILOBYTE; }
break;
case 463:
#line 1607 "p.y"
	{ yyval.number = UNIT_MEGABYTE; }
break;
case 464:
#line 1608 "p.y"
	{ yyval.number = UNIT_GIGABYTE; }
break;
case 465:
#line 1611 "p.y"
	{
                    permset.perm = check_perm(yystack.l_mark[-4].number);
                    addeventaction(&(permset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addperm(&permset);
                  }
break;
case 466:
#line 1618 "p.y"
	{
                    matchset.ignore = FALSE;
                    matchset.match_path = yystack.l_mark[-3].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, yystack.l_mark[0].number);
                    FREE(yystack.l_mark[-3].string); 
                  }
break;
case 467:
#line 1625 "p.y"
	{
                    matchset.ignore = FALSE;
                    matchset.match_path = NULL;
                    matchset.match_string = yystack.l_mark[-3].string;
                    addmatch(&matchset, yystack.l_mark[0].number, 0);
                  }
break;
case 468:
#line 1631 "p.y"
	{
                    matchset.ignore = TRUE;
                    matchset.match_path = yystack.l_mark[0].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, ACTION_IGNORE);
                    FREE(yystack.l_mark[0].string); 
                  }
break;
case 469:
#line 1638 "p.y"
	{
                    matchset.ignore = TRUE;
                    matchset.match_path = NULL;
                    matchset.match_string = yystack.l_mark[0].string;
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 470:
#line 1646 "p.y"
	{
                    matchset.not = FALSE;
                  }
break;
case 471:
#line 1649 "p.y"
	{
                    matchset.not = TRUE;
                  }
break;
case 472:
#line 1655 "p.y"
	{
                    sizeset.operator = yystack.l_mark[-6].number;
                    sizeset.size = ((unsigned long long)yystack.l_mark[-5].number * yystack.l_mark[-4].number);
                    addeventaction(&(sizeset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addsize(&sizeset);
                  }
break;
case 473:
#line 1661 "p.y"
	{
                    sizeset.test_changes = TRUE;
                    addeventaction(&(sizeset).action, yystack.l_mark[0].number, ACTION_IGNORE);
                    addsize(&sizeset);
                  }
break;
case 474:
#line 1668 "p.y"
	{
                    uidset.uid = get_uid(yystack.l_mark[-4].string, 0);
                    addeventaction(&(uidset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    adduid(&uidset);
                    FREE(yystack.l_mark[-4].string);
                  }
break;
case 475:
#line 1674 "p.y"
	{
                    uidset.uid = get_uid(NULL, yystack.l_mark[-4].number);
                    addeventaction(&(uidset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    adduid(&uidset);
                  }
break;
case 476:
#line 1681 "p.y"
	{
                    gidset.gid = get_gid(yystack.l_mark[-4].string, 0);
                    addeventaction(&(gidset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addgid(&gidset);
                    FREE(yystack.l_mark[-4].string);
                  }
break;
case 477:
#line 1687 "p.y"
	{
                    gidset.gid = get_gid(NULL, yystack.l_mark[-4].number);
                    addeventaction(&(gidset).action, yystack.l_mark[-1].number, yystack.l_mark[0].number);
                    addgid(&gidset);
                  }
break;
case 478:
#line 1694 "p.y"
	{ yyval.number = ICMP_ECHO; }
break;
case 479:
#line 1697 "p.y"
	{ yyval.number = 0; }
break;
case 480:
#line 1698 "p.y"
	{ yyval.number = yystack.l_mark[0].number; }
break;
case 481:
#line 1699 "p.y"
	{ yyval.number = yystack.l_mark[-1].number; }
break;
#line 5627 "y.tab.c"
    }
    yystack.s_mark -= yym;
    yystate = *yystack.s_mark;
    yystack.l_mark -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yystack.s_mark = YYFINAL;
        *++yystack.l_mark = yyval;
        if (yychar < 0)
        {
            if ((yychar = YYLEX) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yystack.s_mark, yystate);
#endif
    if (yystack.s_mark >= yystack.s_last && yygrowstack(&yystack))
    {
        goto yyoverflow;
    }
    *++yystack.s_mark = (short) yystate;
    *++yystack.l_mark = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    yyfreestack(&yystack);
    return (1);

yyaccept:
    yyfreestack(&yystack);
    return (0);
}
